# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
import base64
import os

from openstack import exceptions
from openstack import proxy
from otcextensions.sdk.modelartsv2.v2 import dataset as _dataset
from otcextensions.sdk.modelartsv2.v2 import \
    dataset_export_task as _dataset_export_task
from otcextensions.sdk.modelartsv2.v2 import \
    dataset_import_task as _dataset_import_task
from otcextensions.sdk.modelartsv2.v2 import dataset_label as _dataset_label
from otcextensions.sdk.modelartsv2.v2 import dataset_sample as _dataset_sample
from otcextensions.sdk.modelartsv2.v2 import \
    dataset_statistics as _dataset_statistics
from otcextensions.sdk.modelartsv2.v2 import dataset_sync as _dataset_sync
from otcextensions.sdk.modelartsv2.v2 import \
    dataset_version as _dataset_version


class Proxy(proxy.Proxy):
    skip_discovery = True

    def __init__(self, session, *args, **kwargs):
        super(Proxy, self).__init__(session=session, *args, **kwargs)
        self.additional_headers = {
            "Accept": "application/json",
            "Content-type": "application/json",
        }

    # ======== Dataset Management ========
    def datasets(self, **params):
        """List all Datasets.

        :param dict params: Optional query parameters to be sent to limit
            the datasets being returned.

        :returns: A generator of
            (:class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`)
            instances.
        """
        if params.get("limit"):
            params.update(paginated=False)
        return self._list(_dataset.Dataset, **params)

    def create_dataset(self, **attrs):
        """Create a dataset from attributes.

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`,
            comprised of the properties on the Dataset class.
        :returns: The result of dataset creation.
        :rtype: Instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.
        """
        return self._create(_dataset.Dataset, **attrs)

    def get_dataset(self, dataset):
        """Query details of a dataset.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.

        :returns: Instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.
        """
        return self._get(_dataset.Dataset, dataset)

    def find_dataset(self, name_or_id, ignore_missing=True):
        """Find a dataset by name or id.

        :param name_or_id: The name or ID of a dataset.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the dataset does not exist.
            When set to ``True``, no exception will be set when attempting
            to find a nonexistent dataset.

        :returns: Instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.
        """
        return self._find(
            _dataset.Dataset,
            name_or_id,
            ignore_missing=ignore_missing,
        )

    def update_dataset(self, dataset, **attrs):
        """Get details of a dataset.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param dict attrs: Keyword arguments which will be used to update
            a :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`,
            comprised of the properties on the Dataset class.

        :returns: Instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.
        """
        return self._update(_dataset.Dataset, dataset, **attrs)

    def delete_dataset(self, dataset, ignore_missing=False):
        """Delete a dataset.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the dataset does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent dataset.

        returns: ``None``
        """
        return self._delete(
            _dataset.Dataset, dataset, ignore_missing=ignore_missing
        )

    # ======== Dataset Statistics ========

    def get_dataset_statistics(self, dataset):
        """Query Dataset statistics.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`

        :returns: Instance of :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_statistics.DatasetStatistics`.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._get(
            _dataset_statistics.DatasetStatistics,
            uri_dataset_id=dataset.id,
            requires_id=False,
        )

    # ======== Dataset Label Management ========

    def dataset_labels(self, dataset, **params):
        """List all Labels in a dataset.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param dict params: Optional query parameters to be sent to limit
            the instances being returned.

        :returns: A generator of (:class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_label.DatasetLabel`) instances.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._list(
            _dataset_label.DatasetLabel, uri_dataset_id=dataset.id, **params
        )

    def create_dataset_label(self, dataset, **attrs):
        """Create a dataset label from attributes.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_label.DatasetLabel`,
            comprised of the properties on the DatasetLabel class.

        :returns: The results of label creation.
        :rtype: Instance of :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_label.DatasetLabel`.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._create(
            _dataset_label.DatasetLabel, uri_dataset_id=dataset.id, **attrs
        )

    def update_dataset_labels(self, dataset, labels=[], **attrs):
        """Modify dataset labels in batches.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.
        :param labels: List of Labels to update.
        :param dict attrs: Keyword arguments which will be used to update
            a :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_label.DatasetLabel`,
            comprised of the properties on the DatasetLabel class.

        :returns: Instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset_label.DatasetLabel`
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        obj = self._get_resource(
            _dataset_label.DatasetLabel, None, uri_dataset_id=dataset.id
        )
        labels = labels or attrs.get("labels")
        return obj.update_labels(self, labels)

    def delete_dataset_labels(
        self, dataset, labels=[], delete_policy=None, **attrs
    ):
        """Delete dataset labels in batches.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.
        :param labels: List of labels dict to delete.
        :param dict attrs: Keyword arguments which will be used to delete a
            :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_label.DatasetLabel`,
            comprised of the properties on the DatasetLabel class.

        :returns: ``None``
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        obj = self._get_resource(
            _dataset_label.DatasetLabel, None, uri_dataset_id=dataset.id
        )
        labels = labels or attrs.get("labels")
        return obj.delete_labels(self, labels, delete_policy)

    # ======== Dataset Version Management ========

    def dataset_versions(self, dataset, **params):
        """List all versions of a dataset.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.

        :returns: a generator of (:class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_version.DatasetVersion`) instances.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._list(
            _dataset_version.DatasetVersion, uri_dataset_id=dataset.id
        )

    def create_dataset_version(self, dataset, **attrs):
        """Create a dataset version from attributes.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_version.DatasetVersion`,
            comprised of the properties on the DatasetVersion class.

        :returns: The results of dataset creation.
        :rtype: Instance of :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_version.DatasetVersion`.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._create(
            _dataset_version.DatasetVersion,
            uri_dataset_id=dataset.id,
            **attrs,
        )

    def get_dataset_version(self, dataset, dataset_version):
        """Get details of a dataset version.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.
        :param dataset_version: Dataset version id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset_version.DatasetVersion`.

        :returns: Instance of :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_version.DatasetVersion`.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._get(
            _dataset_version.DatasetVersion,
            dataset_version,
            uri_dataset_id=dataset.id,
        )

    def delete_dataset_version(
        self, dataset, dataset_version, ignore_missing=False
    ):
        """Delete a dataset version

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param dataset_version: Dataset version id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset_version.DatasetVersion`
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the group does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent dataset.

        :returns: ``None``
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._delete(
            _dataset_version.DatasetVersion,
            dataset_version,
            uri_dataset_id=dataset.id,
            ignore_missing=ignore_missing,
        )

    # ======== Dataset Sample Management ========

    def dataset_samples(self, dataset, **params):
        """List all samples in a Dataset.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param dict params: Optional query parameters to be sent to limit
            the dataset samples being returned.
        :returns: a generator of (:class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_sample.DatasetSample`) instances.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._list(
            _dataset_sample.DatasetSample,
            uri_dataset_id=dataset.id,
            **params,
            paginated=False,
        )

    def add_dataset_samples(
        self,
        dataset,
        directory_path=None,
        file_path=None,
        sample_type=None,
        labels=[],
        metadata={},
        data_source={},
    ):
        """Upload samples to a dataset.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.
        :param dict attrs: Keyword arguments which will be used to create
            upload samples to a dataset and obtain instance of a
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset_sample.DatasetSample` # noqa: E501

        :returns: The result after uploading the dataset samples.
        :rtype: Instance of :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_sample.DatasetSample`.
        """

        def get_directory_size(dir_path, file_type=None):
            total_size = 0
            if not os.path.isdir(dir_path):
                raise exceptions.SDKException(
                    f"Error: {dir_path} is not a valid directory."
                )
            files = [
                os.path.join(dir_path, f)
                for f in os.listdir(dir_path)
                if os.path.isfile(os.path.join(dir_path, f))
            ]
            if not files:
                raise exceptions.SDKException(
                    f"The directory {dir_path} is empty."
                )
            for file_name in files:
                file_path = os.path.join(dir_path, file_name)
                total_size += os.path.getsize(file_path)
            if total_size == 0:
                raise exceptions.SDKException("Directory is empty")
            return total_size, files

        if file_path:
            file_size = os.path.getsize(file_path) / (1024 * 1024)
            if file_size > 7.5:
                raise exceptions.SDKException("File size is > 7.5 Mb")
            files = [file_path]
            # with Image.open(file_path) as image:
            #     with io.BytesIO() as byte_stream:
            #         image.save(byte_stream, format="JPEG")
            #         image_bytes = byte_stream.getvalue()
            # sample = {
            #     "name": os.path.split(file_path)[1],
            #     "data": base64.b64encode(image_bytes).decode("utf-8"),
            # }

        if not file_path and directory_path:
            _, files = get_directory_size(directory_path)
            # size, files = get_directory_size(directory_path)
            # if size / (1024 * 1024) > 7.5:
            #     raise exceptions.SDKException("Files size is > 7.5 Mb")

        count = 0
        samples = []
        files_size = 0
        for file_path in files:
            sample = {}
            count = count + 1
            files_size = files_size + os.path.getsize(file_path) / (
                1024 * 1024
            )
            if files_size > 7.5:
                break
            with open(file_path, "rb") as file:
                sample = {
                    "name": os.path.split(file_path)[1],
                    "data": base64.b64encode(file.read()).decode(),
                }
            if labels:
                sample.update(labels=labels)
            if data_source:
                sample.update(data_source=data_source)
            if sample_type:
                sample.update(sample_type=sample_type)
            if metadata:
                sample.update(metadata=metadata)
            samples.append(sample)
        attrs = {}
        if samples:
            attrs.update(samples=samples)
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._create(
            _dataset_sample.DatasetSample, uri_dataset_id=dataset.id, **attrs
        )

    def delete_dataset_samples(self, dataset, samples=[], delete_source=False):
        """Delete dataset samples in batches.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param samples: List of sample ID(s) to delete.
        :param bool delete_source: When set to ``True``
            sample source files will be also deleted.

        :returns: ``None``
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        dataset_sample = self._get_resource(
            _dataset_sample.DatasetSample, None
        )
        return dataset_sample.delete_samples(
            self, dataset.id, samples, delete_source
        )

    def get_dataset_sample(self, dataset, dataset_sample):
        """Get details of the dataset sample.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param dataset_sample: Dataset sample id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset_sample.DatasetSample`

        :returns: Instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset_sample.DatasetSample`
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._get(
            _dataset_sample.DatasetSample,
            dataset_sample,
            uri_dataset_id=dataset.id,
        )

    # ======== Dataset Import Task Management ========

    def dataset_import_tasks(self, dataset, **params):
        """List all dataset import tasks.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param dict params: Optional query parameters to be sent to limit
            the dataset import tasks being returned.

        :returns: A generator of (:class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_import_task.DatasetImportTask`).
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._list(
            _dataset_import_task.DatasetImportTask,
            uri_dataset_id=dataset.id,
            **params,
        )

    def create_dataset_import_task(self, dataset, **attrs):
        """Create a data import task from attributes.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_import_task.DatasetImportTask`,
            comprised of the properties on the DatasetImportTask class.

        :returns: The results of dataset import task.
        :rtype: Instance of :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_import_task.DatasetImportTask`.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._create(
            _dataset_import_task.DatasetImportTask,
            uri_dataset_id=dataset.id,
            **attrs,
        )

    def get_dataset_import_task(self, dataset, import_task):
        """Get details of a dataset import task.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param import_task: Import task id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_import_task.DatasetImportTask`.

        :returns: Instance of :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_import_task.DatasetImportTask`.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._get(
            _dataset_import_task.DatasetImportTask,
            import_task,
            uri_dataset_id=dataset.id,
        )

    # ======== Dataset Export Task Management ========

    def dataset_export_tasks(self, dataset, **params):
        """List all dataset export tasks.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param dict params: Optional query parameters to be sent to limit
            the dataset export tasks being returned.

        :returns: A generator of
            (:class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_export_task.DatasetExportTask`) instances.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._list(
            _dataset_export_task.DatasetExportTask,
            uri_dataset_id=dataset.id,
            **params,
        )

    def create_dataset_export_task(self, dataset, **attrs):
        """Create a data import task from attributes.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_export_task.DatasetExportTask`,
            comprised of the properties on the DatasetExportTask class.

        :returns: The results of dataset export task.
        :rtype: Instance of :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_export_task.DatasetExportTask`.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._create(
            _dataset_export_task.DatasetExportTask,
            uri_dataset_id=dataset.id,
            **attrs,
        )

    def get_dataset_export_task(self, dataset, export_task):
        """Get details of a dataset export task.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`
        :param export_task: Export Task id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_export_task.DatasetExportTask`.

        :returns: Instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.
            dataset_export_task.DatasetExportTask`.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._get(
            _dataset_export_task.DatasetExportTask,
            export_task,
            uri_dataset_id=dataset.id,
        )

    # ======== Dataset Synchronization Task Management ========

    def dataset_sync(self, dataset):
        """Synchronize samples and labeling information
            from the input dataset path to the dataset.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.

        :returns: Instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset_sync.DatasetSync`.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._create(
            _dataset_sync.DatasetSync, uri_dataset_id=dataset.id
        )

    def get_dataset_sync_status(self, dataset):
        """Query Dataset sync task status.

        :param dataset: Dataset id or an instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset.Dataset`.

        :returns: Instance of
            :class:`~otcextensions.sdk.modelartsv2.v2.dataset_sync.DatasetSync`.
        """
        dataset = self._get_resource(_dataset.Dataset, dataset)
        return self._get(
            _dataset_sync.DatasetSync, "status", uri_dataset_id=dataset.id
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import _log
from otcextensions.sdk import dnsmixin


_logger = _log.setup_logging('openstack')


class NameServer(dnsmixin.DNSProxyMixin, resource.Resource):
    resource_key = 'nameserver'
    resources_key = 'nameservers'
    base_path = '/zones/%(zone_id)s/nameservers'

    # capabilities
    allow_create = False
    allow_list = True
    allow_get = False
    allow_delete = False
    allow_update = False

    #: Properties
    #: The ID of zone using this name-server
    zone_id = resource.URI('zone_id')
    #: NameServer priority
    priority = resource.Body('priority')
    #: NameServer private DNS address
    address = resource.Body('address')
    #: NameServer public DNS address
    hostname = resource.Body('hostname')



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Create a model from attributes."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")

attrs = {
    "model_name": "mnist",
    "model_version": "1.0.0",
    "source_location": "https://models.obs.xxxx.com/mnist",
    "source_job_id": "55",
    "source_job_version": "V100",
    "model_type": "TensorFlow",
    "runtime": "python2.7",
    "description": "mnist model",
    "execution_code": "https://testmodel.obs.xxxx.com/customize_service.py",
    "input_params": [
        {
            "url": "/v1/xxx/image",
            "protocol": "http",
            "method": "post",
            "param_name": "image_url",
            "param_type": "string",
            "min": 0,
            "max": 9,
            "param_desc": "http://test/test.jpeg",
        }
    ],
    "output_params": [
        {
            "url": "/v1/xxx/image",
            "protocol": "http",
            "method": "post",
            "param_name": "face_location",
            "param_type": "box",
            "param_desc": "face_location param value description",
        }
    ],
    "dependencies": [
        {
            "installer": "pip",
            "packages": [
                {
                    "package_name": "numpy",
                    "package_version": "1.5.0",
                    "restraint": "ATLEAST",
                }
            ],
        }
    ],
    "model_algorithm": "object_detection",
    "model_metrics": '{"f1":0.52381,"recall":0.666667,\
        "precision":0.466667,"accuracy":0.625}',
    "apis": [
        {
            "url": "/v1/xxx/image",
            "protocol": "http",
            "method": "post",
            "input_params": {
                "type": "object",
                "properties": {"image_url": {"type": "string"}},
            },
            "output_params": {
                "type": "object",
                "properties": {"face_location": {"type": "box"}},
            },
        }
    ],
}

model = conn.modelartsv1.create_model(**attrs)
print(model)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy
from otcextensions.sdk.imsv2.v2 import image as _image


class Proxy(proxy.Proxy):
    skip_discovery = True

    def create_image(self, **attrs):
        """Create a new image with attrs

        :param dict attrs: Keyword arguments which will be used to create a
            :class:`~otcextensions.sdk.imsv2.v2.image.Image`
        """
        base_path = '/cloudimages/action'
        return self._create(_image.Image, base_path=base_path, **attrs)

    def images(self, **attrs):
        """Retrieve a generator of images

        :param dict query: Optional query parameters to be sent to limit the
            resources being returned.

        :returns: A generator of zone
            :class:`~otcextensions.sdk.imsv2.v2.image.Image` instances
        """
        return self._list(_image.Image, paginated=False, **attrs)

    def update_image(self, image_id, **attrs):
        """Update an image

        :param image: The value can be either the ID of a image or a
            :class:`~otcextensions.sdk.imsv2.v2.image.Image` instance.
        :param dict attrs: The attributes to update of the image represented
            by ``image``.

        :returns: The updated image.

        :rtype: :class:`~otcextensions.sdk.imsv2.v2.image.Image`
        """
        image = _image.Image()
        return image.update_image_details(self, image_id=image_id, **attrs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log

from otcextensions.tests.functional import base

_logger = _log.setup_logging('openstack')


class TestClusterNodes(base.BaseFunctionalTest):

    TEST_CLUSTER = '5a66a449-668c-492f-8c33-5cdbdeaadd2e'

    def setUp(self):
        super(TestClusterNodes, self).setUp()
        self.cce = self.conn.cce

    def test_list_nodes(self):
        cluster = self.cce.get_cluster(self.TEST_CLUSTER)
        nodes = list(self.cce.cluster_nodes(cluster))

        self.assertGreaterEqual(len(nodes), 0)

    def test_get_node(self):
        cluster = self.cce.get_cluster(self.TEST_CLUSTER)
        nodes = list(self.cce.cluster_nodes(cluster))
        if len(nodes) > 0:
            node = self.cce.get_cluster_node(cluster, nodes[0].id)
            self.assertIsNotNone(node)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
import mock

# from otcextensions.common import sdk_utils
from otcextensions.osclient.anti_ddos.v1 import floating_ip
from otcextensions.tests.unit.osclient.anti_ddos.v1 import fakes


class TestListFloatingIP(fakes.TestAntiDDoS):

    objects = fakes.FakeFloatingIP.create_multiple(3)

    columns = (
        'floating_ip_address', 'floating_ip_id', 'network_type', 'status'
    )

    data = []

    for s in objects:
        data.append((
            s.floating_ip_address,
            s.floating_ip_id,
            s.network_type,
            s.status
        ))

    def setUp(self):
        super(TestListFloatingIP, self).setUp()

        self.cmd = floating_ip.ListFloatingIP(self.app, None)

        self.client.floating_ips = mock.Mock()
        self.client.api_mock = self.client.floating_ips

    def test_list(self):
        arglist = [
        ]

        verifylist = [
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))

    def test_list_query(self):
        arglist = [
            '--ip', 'ip_pref',
            '--status', 'normal'
        ]

        verifylist = [
            ('ip', 'ip_pref'),
            ('status', 'normal')
        ]

        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self.objects
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_once_with(
            ip='ip_pref',
            status='normal'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))


class TestShow(fakes.TestAntiDDoS):

    _data = fakes.FakeFloatingIP.create_one()

    columns = (
        'app_type_id', 'cleaning_access_pos_id', 'floating_ip_address',
        'floating_ip_id', 'http_request_pos_id', 'id', 'is_enable_l7',
        'network_type', 'status')

    data = (
        _data.app_type_id,
        _data.cleaning_access_pos_id,
        _data.floating_ip_address,
        _data.floating_ip_id,
        _data.http_request_pos_id,
        _data.id,
        _data.is_enable_l7,
        _data.network_type,
        _data.status
    )

    def setUp(self):
        super(TestShow, self).setUp()

        self.cmd = floating_ip.ShowFloatingIP(self.app, None)

        self.client.get_floating_ip_policies = mock.Mock()
        self.client.api_mock = self.client.get_floating_ip_policies

    def test_show_default(self):
        arglist = [
            'name_or_id',
        ]
        verifylist = [
            ('floating_ip_id', 'name_or_id'),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            'name_or_id'
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)


class TestSetFloatingIP(fakes.TestAntiDDoS):

    _data = fakes.FakeFloatingIP.create_one()

    columns = (
        'app_type_id', 'cleaning_access_pos_id', 'floating_ip_address',
        'floating_ip_id', 'http_request_pos_id', 'id', 'is_enable_l7',
        'network_type', 'status')

    data = (
        _data.app_type_id,
        _data.cleaning_access_pos_id,
        _data.floating_ip_address,
        _data.floating_ip_id,
        _data.http_request_pos_id,
        _data.id,
        _data.is_enable_l7,
        _data.network_type,
        _data.status
    )

    def setUp(self):
        super(TestSetFloatingIP, self).setUp()

        self.cmd = floating_ip.SetFloatingIP(self.app, None)

        self.client.update_floating_ip_policies = mock.Mock()
        self.client.api_mock = self.client.update_floating_ip_policies

    def test_default(self):
        arglist = [
            'ip_id',
            '--enable_l7',
            '--traffic_pos_id', '1',
            '--http_request_pos_id', '2',
            '--cleaning_access_pos_id', '3',
            '--app_type_id', '0',
        ]
        verifylist = [
            ('floating_ip_id', 'ip_id'),
            ('enable_l7', True),
            ('traffic_pos_id', 1),
            ('http_request_pos_id', 2),
            ('cleaning_access_pos_id', 3),
            ('app_type_id', 0),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            floating_ip='ip_id',
            traffic_pos_id=1,
            http_request_pos_id=2,
            cleaning_access_pos_id=3,
            is_enable_l7=True,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)

    def test_enable_false(self):
        arglist = [
            'ip_id',
            '--disable_l7',
            '--traffic_pos_id', '1',
            '--http_request_pos_id', '2',
            '--cleaning_access_pos_id', '3',
            '--app_type_id', '0',
        ]
        verifylist = [
            ('floating_ip_id', 'ip_id'),
            ('disable_l7', True),
            ('traffic_pos_id', 1),
            ('http_request_pos_id', 2),
            ('cleaning_access_pos_id', 3),
            ('app_type_id', 0),
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.api_mock.side_effect = [
            self._data
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.api_mock.assert_called_with(
            floating_ip='ip_id',
            traffic_pos_id=1,
            http_request_pos_id=2,
            cleaning_access_pos_id=3,
            is_enable_l7=False,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack import resource
from openstack import utils
from openstack import exceptions


class PredefinedTag(resource.Resource):
    resource_key = 'predefine_tag'
    base_path = '/predefine_tags'

    _query_mapping = resource.QueryParameters(
        'key', 'value', 'limit', 'marker', 'order_field', 'order_method')

    #: Properties
    marker = resource.Body('marker')
    total_count = resource.Body('total_count')
    tags = resource.Body('tags', type=list)
    action = resource.Body('action')
    old_tag = resource.Body('old_tag', type=dict)
    new_tag = resource.Body('new_tag', type=dict)

    #: Allow to create operation for this resource.
    allow_create = True
    #: Allow get operation for this resource.
    allow_fetch = False
    #: Allow update operation for this resource.
    allow_commit = True
    #: Allow to delete operation for this resource.
    allow_delete = True
    #: Allow list operation for this resource.
    allow_list = True
    #: Allow head operation for this resource.
    allow_head = False
    #: Allow patch operation for this resource.
    allow_patch = False

    #: Commits happen without header or body being dirty.
    allow_empty_commit = False

    #: Method for committing a resource (PUT, PATCH, POST)
    commit_method = "PUT"
    #: Method for creating a resource (POST, PUT)
    create_method = "POST"

    requires_id = False

    def _action(self, session, request_body):
        url = utils.urljoin(self.base_path, 'action')
        response = session.post(url, json=request_body)
        exceptions.raise_from_response(response)

    def add_tag(self, session, key, value):
        request_body = {
            "action": "create",
            "tags": [{
                "key": key,
                "value": value
            }]
        }
        self._action(session, request_body)

    def delete_tag(self, session, key, value):
        request_body = {
            "action": "delete",
            "tags": [{
                "key": key,
                "value": value
            }]
        }
        self._action(session, request_body)

    def _prepare_request_body(
        self,
        patch,
        prepend_key,
        *,
        resource_request_key=None,
    ):
        return self._body.dirty



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
import mock

from otcextensions.osclient.auto_scaling.v1 import activity
from otcextensions.tests.unit.osclient.auto_scaling.v1 import fakes


class TestAutoScalingActivity(fakes.TestAutoScaling):

    def setUp(self):
        super(TestAutoScalingActivity, self).setUp()
        self.client = self.app.client_manager.auto_scaling


class TestListAutoScalingActivity(TestAutoScalingActivity):

    _activities = fakes.FakeActivity.create_multiple(3)
    _group = fakes.FakeGroup.create_one()

    columns = (
        'ID', 'status', 'description',
        'instance_value', 'desire_value',
        'start_time', 'end_time')

    data = []

    for s in _activities:
        data.append((
            s.id,
            s.status,
            s.description,
            s.instance_value,
            s.desire_value,
            s.start_time,
            s.end_time
        ))

    def setUp(self):
        super(TestListAutoScalingActivity, self).setUp()

        self.cmd = activity.ListAutoScalingActivityLogs(self.app, None)

        self.client.find_group = mock.Mock()
        self.client.activities = mock.Mock()

    def test_list_default(self):
        arglist = [
            '--start-time', '2200-01-01T00:00:00Z',
            '--end-time', '2200-01-02T00:00:00Z',
            '--limit', '14',
            '--group', 'group1'
        ]

        verifylist = [
            ('start_time', '2200-01-01T00:00:00Z'),
            ('end_time', '2200-01-02T00:00:00Z'),
            ('limit', 14),
            ('group', 'group1')
        ]
        # Verify cm is triggereg with default parameters
        parsed_args = self.check_parser(self.cmd, arglist, verifylist)

        # Set the response
        self.client.find_group.side_effect = [
            self._group
        ]
        self.client.activities.side_effect = [
            self._activities
        ]

        # Trigger the action
        columns, data = self.cmd.take_action(parsed_args)

        self.client.find_group.assert_called_once_with(
            'group1', ignore_missing=False)
        self.client.activities.assert_called_once_with(
            end_time='2200-01-02T00:00:00Z',
            start_time='2200-01-01T00:00:00Z',
            group=self._group.id,
            limit=14,
        )

        self.assertEqual(self.columns, columns)
        self.assertEqual(self.data, list(data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.dns.v2 import zone


FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE = {
    "id": FAKE_ID,
    "name": "example.com.",
    "description": "This is an example zone.",
    "email": "xx@example.com",
    "ttl": 300,
    "serial": 0,
    "masters": [],
    "status": "ACTIVE",
    "links": {
        "self": "https://Endpoint/v2/zones/ff8080825b8fc86c015b94bc6f8712c3"
    },
    "pool_id": "ff8080825ab738f4015ab7513298010e",
    "project_id": "e55c6f3dc4e34c9f86353b664ae0e70c",
    "zone_type": "private",
    "created_at": "2017-04-22T08:17:08.997",
    "updated_at": "2017-04-22T08:17:09.997",
    "record_num": 2,
    "routers": [
        {
            "status": "ACTIVE",
            "router_id": "19664294-0bf6-4271-ad3a-94b8c79c6558",
            "router_region": "xx"
        },
        {
            "status": "ACTIVE",
            "router_id": "f0791650-db8c-4a20-8a44-a06c6e24b15b",
            "router_region": "xx"
        }
    ]

}


class TestZone(base.TestCase):

    def setUp(self):
        super(TestZone, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()

    def test_basic(self):
        sot = zone.Zone()

        self.assertEqual('/zones', sot.base_path)
        self.assertEqual('zones', sot.resources_key)
        self.assertIsNone(sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):

        sot = zone.Zone(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['email'], sot.email)
        self.assertEqual(EXAMPLE['ttl'], sot.ttl)
        self.assertEqual(EXAMPLE['serial'], sot.serial)
        self.assertEqual(EXAMPLE['masters'], sot.masters)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['pool_id'], sot.pool_id)
        self.assertEqual(EXAMPLE['project_id'], sot.project_id)
        self.assertEqual(EXAMPLE['zone_type'], sot.zone_type)
        self.assertEqual(EXAMPLE['created_at'], sot.created_at)
        self.assertEqual(EXAMPLE['updated_at'], sot.updated_at)
        self.assertEqual(EXAMPLE['record_num'], sot.record_num)
        self.assertEqual(len(EXAMPLE['routers']), len(sot.routers))

    def test_associate_router(self):
        sot = zone.Zone(**EXAMPLE)

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot.associate_router(
            self.sess, router_id=1, router_region='a'
        )

        self.sess.post.assert_called_once_with(
            'zones/%s/associaterouter' % FAKE_ID,
            json={'router': {'router_id': 1, 'router_region': 'a'}}
        )

    def test_disassociate_router(self):
        sot = zone.Zone(**EXAMPLE)

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot.disassociate_router(
            self.sess, router_id=1
        )

        self.sess.post.assert_called_once_with(
            'zones/%s/disassociaterouter' % FAKE_ID,
            json={'router': {'router_id': 1}}
        )



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CCE Cluster Nodes v2 action implementations'''
import logging

from osc_lib import utils
from osc_lib.cli import parseractions
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def _flatten_cluster_node(obj):
    """Flatten the structure of the cluster node into a single dict
    """
    data = {
        'id': obj.id,
        'name': obj.name,
        'private_ip': obj.status.private_ip,
        'public_ip': obj.status.floating_ip,
        'status': obj.status.status,
        'flavor': obj.spec.flavor,
        'ssh_key': obj.spec.login.get('sshKey', None),
        'availability_zone': obj.spec.availability_zone,
        'operating_system': obj.spec.os,
        'root_volume_type': obj.spec.root_volume.type,
        'root_volume_size': obj.spec.root_volume.size
    }

    i = 1
    for item in obj.spec.data_volumes:
        data['data_volume_type_' + str(i)] = item.get('type')
        data['data_volume_size_' + str(i)] = item.get('size')
        i += 1

    return data


class ListCCEClusterNode(command.Lister):
    _description = _('List CCE Cluster Nodes')
    columns = ('ID', 'name', 'private_ip', 'public_ip',
               'status')

    def get_parser(self, prog_name):
        parser = super(ListCCEClusterNode, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('The ID or name of the cluster.')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        cluster = client.find_cluster(parsed_args.cluster,
                                      ignore_missing=False)

        data = client.cluster_nodes(cluster.id)

        table = (self.columns,
                 (utils.get_dict_properties(
                     _flatten_cluster_node(s), self.columns,
                 ) for s in data))
        return table


class ShowCCEClusterNode(command.ShowOne):
    _description = _('Show single Cluster node details')
    columns = (
        'ID',
        'name',
        'flavor',
        'private_ip',
        'public_ip',
        'availability_zone',
        'ssh_key',
        'status',
        'root_volume_type',
        'root_volume_size',)

    def get_parser(self, prog_name):
        parser = super(ShowCCEClusterNode, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID of the cluster.')
        )
        parser.add_argument(
            'node',
            metavar='<node>',
            help=_('ID of the cluster node.')
        )
        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.cce

        cluster = client.find_cluster(parsed_args.cluster,
                                      ignore_missing=False)

        obj = client.find_cluster_node(
            cluster=cluster.id,
            node=parsed_args.node
        )

        flat_data = _flatten_cluster_node(obj)
        columns = tuple(flat_data)

        data = utils.get_dict_properties(
            flat_data,
            columns)

        return (columns, data)


class DeleteCCEClusterNode(command.Command):
    _description = _('Delete CCE Cluster nodes')

    def get_parser(self, prog_name):
        parser = super(DeleteCCEClusterNode, self).get_parser(prog_name)
        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID of the cluster.')
        )
        parser.add_argument(
            'node',
            metavar='<node>',
            nargs='+',
            help=_('Name of the cluster node.')
        )
        return parser

    def take_action(self, parsed_args):

        if parsed_args.cluster and parsed_args.node:
            client = self.app.client_manager.cce
            cluster = client.find_cluster(parsed_args.cluster,
                                          ignore_missing=False)
            for node in parsed_args.node:
                obj = client.find_cluster_node(
                    cluster=cluster.id,
                    node=node)
                client.delete_cluster_node(
                    cluster=cluster.id,
                    node=obj.id,
                    ignore_missing=False)


class CreateCCEClusterNode(command.Command):
    _description = _('Create CCE Cluster Node')

    columns = (
        'ID',
        'name',
        'flavor',
        'private_ip',
        'public_ip',
        'availability_zone',
        'ssh_key',
        'status',
        'root_volume_type',
        'root_volume_size')

    def get_parser(self, prog_name):
        parser = super(CreateCCEClusterNode, self).get_parser(prog_name)

        parser.add_argument(
            'cluster',
            metavar='<cluster>',
            help=_('ID or name of the CCE cluster.')
        )
        parser.add_argument(
            '--name',
            metavar='<name>',
            help=_('Name of the CCE node.\nThe clustername and a random '
                   'number is used to create a node name by default.')
        )
        parser.add_argument(
            '--annotation',
            metavar='<key_name>=<value_name>',
            action=parseractions.KeyValueAction,
            dest='annotations',
            help=_('Annotations in form of key, value pairs.'
                   'Repeat option for multiple tags.\n'
                   'Example: '
                   '--annotation keyname1=valuename1 '
                   '--annotation keyname2=valuename2')
        )
        parser.add_argument(
            '--availability-zone',
            metavar='<availability_zone>',
            required=True,
            help=_('Availability zone to place server in.')
        )
        parser.add_argument(
            '--bandwidth',
            metavar='<bandwidth>',
            type=int,
            help=_('Bandwidth of the floating ip being created. fip-count'
                   'must be specified if bandwidth is used.')
        )
        parser.add_argument(
            '--count',
            metavar='<count>',
            default=1,
            type=int,
            help=_('Count of the cluster nodes to be created.\n'
                   'Default: 1.')
        )
        parser.add_argument(
            '--data-volume',
            metavar='volumetype=<volumetype>,size=<disksize>,'
                    'encrypted=<True|False>,cmk_id=<cmk_id>',
            action=parseractions.MultiKeyValueAction,
            dest='data_volumes',
            required_keys=['volumetype', 'size'],
            optional_keys=['encrypted', 'cmk_id'],
            help=_('Example: '
                   '--data-volume volumetype=SATA,size=100,encrypted=True,'
                   'cmk_id=12345qwertz \n'
                   'Repeat option for multiple data volumes.\n'
                   'Default: --data-volume volumetype=SATA,size=100')
        )
        parser.add_argument(
            '--dedicated-host',
            metavar='<dedicated_host>',
            help=_('Name or ID of the Dedicated Host to which nodes will.'
                   'be scheduled.')
        )
        parser.add_argument(
            '--ecs-group',
            metavar='<group_id>',
            help=_('ID of the ECS group where the CCE node can belong to.')
        )
        parser.add_argument(
            '--fault-domain',
            metavar='<fault_domain>',
            help=_('The node is created in the specified fault domain.')
        )
        parser.add_argument(
            '--flavor',
            metavar='<flavor>',
            required=True,
            help=_('The flavor for the node.')
        )
        parser.add_argument(
            '--floating-ip',
            metavar='<floating_ip>',
            action='append',
            help=_('Address or ID of the existing floating IP to be '
                   'attached to the new node.\n'
                   'Repeat option for multiple IPs to be attached to '
                   'multiple nodes.')
        )
        parser.add_argument(
            '--fip-count',
            metavar='<fip_count>',
            type=int,
            help=_('Count of floating IP addresses being attached to one '
                   'or more nodes.\nThe parameter must be used together with'
                   'bandwidth.')
        )
        parser.add_argument(
            '--k8s-tag',
            metavar='<key_name>=<value_name>',
            action=parseractions.KeyValueAction,
            dest='k8s_tags',
            help=_('Kubernetes tags in form of key, value pairs. Repeat '
                   'option for multiple tags.\n'
                   'Example:\n'
                   '--k8s-tag keyname1=valuename1 '
                   '--k8s-tag keyname2=valuename2')
        )
        parser.add_argument(
            '--label',
            metavar='<key_name>=<value_name>',
            action=parseractions.KeyValueAction,
            dest='labels',
            help=_('Option labels in form of key, value pairs. Repeat '
                   'option for multiple labels.\n'
                   'Example:\n'
                   '--label keyname1=valuename1 '
                   '--label keyname2=valuename2')
        )
        parser.add_argument(
            '--lvm-config',
            metavar='<lvm_config>',
            help=_('ConfigMap of the Docker data disk.')
        )
        parser.add_argument(
            '--max-pods',
            metavar='<max_pods>',
            type=int,
            help=_('Maximum number of pods on the node')
        )
        parser.add_argument(
            '--network',
            metavar='<network>',
            required=True,
            help=_('ID or name of the network where the node will be '
                   'created.')
        )
        parser.add_argument(
            '--node-image-id',
            metavar='<node_image_id>',
            help=_('ID of a custom image used in a bare metal scenario.')
        )
        parser.add_argument(
            '--os',
            metavar='<operating_system>',
            help=_('Operating system of the cluster node.')
        )
        parser.add_argument(
            '--postinstall-script',
            metavar='<postinstall_script>',
            help=_('Base64 encoded post installation script.')
        )
        parser.add_argument(
            '--preinstall-script',
            metavar='<preinstall_script>',
            help=_('Base64 encoded pre installation script.')
        )
        parser.add_argument(
            '--root-volume-size',
            metavar='<root_volume_size>',
            type=int,
            default=40,
            help=_('Root volume size in GB.')
        )
        parser.add_argument(
            '--root-volume-type',
            metavar='<root_volume_type>',
            choices=['SAS', 'SATA', 'SSD'],
            default='SATA',
            help=_('Root volume type.\n'
                   'Options: SAS, SATA, SSD.')
        )
        parser.add_argument(
            '--ssh-key',
            metavar='<ssh_key>',
            required=True,
            help=_('SSH Key used to create the node.')
        )
        parser.add_argument(
            '--tag',
            metavar='key=<keyname1>,value=<value1>',
            action=parseractions.MultiKeyValueAction,
            dest='tags',
            required_keys=['key', 'value'],
            help=_('List of tags used to build UI labels. Repeat option for '
                   'multiple tags.\n'
                   'Example:\n'
                   '--tag key=mykey1,value=myvalue1')
        )
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for the instance to become active')
        )
        parser.add_argument(
            '--wait-interval',
            metavar='<interval>',
            type=int,
            default=5,
            help=_('Check interval in seconds for successful creation check'
                   'when param wait is True.')
        )
        parser.add_argument(
            '--wait-timeout',
            metavar='<timeout>',
            type=int,
            default=3600,
            help=_('Maximum time in seconds to wait for successful creation'
                   'when param wait is True.')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}

        # mandatory
        attrs['availability_zone'] = parsed_args.availability_zone
        attrs['cluster'] = parsed_args.cluster
        attrs['count'] = parsed_args.count
        attrs['flavor'] = parsed_args.flavor
        attrs['network'] = parsed_args.network
        attrs['ssh_key'] = parsed_args.ssh_key

        # optional
        if parsed_args.annotations:
            attrs['annotations'] = parsed_args.annotations
        if parsed_args.bandwidth:
            attrs['bandwidth'] = parsed_args.bandwidth
        if parsed_args.count:
            attrs['count'] = parsed_args.count
        if parsed_args.data_volumes:
            attrs['data_volumes'] = parsed_args.data_volumes
        if parsed_args.dedicated_host:
            attrs['dedicated_host'] = parsed_args.dedicated_host
        if parsed_args.ecs_group:
            attrs['ecs_group'] = parsed_args.ecs_group
        if parsed_args.fault_domain:
            attrs['fault_domain'] = parsed_args.fault_domain
        if parsed_args.floating_ip:
            attrs['floating_ips'] = parsed_args.floating_ip
        if parsed_args.fip_count:
            attrs['fip_count'] = parsed_args.fip_count
        if parsed_args.k8s_tags:
            attrs['k8s_tags'] = parsed_args.k8s_tags
        if parsed_args.labels:
            attrs['labels'] = parsed_args.labels
        if parsed_args.max_pods:
            attrs['max_pods'] = parsed_args.max_pods
        if parsed_args.name:
            attrs['name'] = parsed_args.name
        if parsed_args.network:
            attrs['network'] = parsed_args.network
        if parsed_args.node_image_id:
            attrs['node_image_id'] = parsed_args.node_image_id
        if parsed_args.os:
            attrs['os'] = parsed_args.os
        if parsed_args.postinstall_script:
            attrs['postinstall_script'] = parsed_args.postinstall_script
        if parsed_args.preinstall_script:
            attrs['preinstall_script'] = parsed_args.preinstall_script
        if parsed_args.root_volume_size:
            attrs['root_volume_size'] = parsed_args.root_volume_size
        if parsed_args.root_volume_type:
            attrs['root_volume_type'] = parsed_args.root_volume_type
        if parsed_args.tags:
            attrs['tags'] = parsed_args.tags
        if not parsed_args.wait:
            attrs['wait'] = False
        if parsed_args.wait_interval:
            attrs['wait_interval'] = parsed_args.wait_interval
        if parsed_args.wait_timeout:
            attrs['wait_timeout'] = parsed_args.wait_timeout

        obj = self.app.client_manager.sdk_connection.create_cce_cluster_node(
            **attrs)

        flat_data = _flatten_cluster_node(obj)
        columns = tuple(flat_data)

        data = utils.get_dict_properties(
            flat_data,
            columns)

        return (columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from keystoneauth1 import adapter
from openstack.tests.unit import base

from otcextensions.sdk.vpc.v1 import subnet

IDENTIFIER = 'ID'
EXAMPLE = {
    'id': '4779ab1c-7c1a-44b1-a02e-93dfc361b32d',
    'name': 'subnet',
    'description': '',
    'cidr': '192.168.20.0/24',
    'dnsList': [
        '8.8.8.8',
        '1.1.1.1'
    ],
    'status': 'ACTIVE',
    'vpc_id': '3ec3b33f-ac1c-4630-ad1c-7dba1ed79d85',
    'gateway_ip': '192.168.20.1',
    'dhcp_enable': True,
    'primary_dns': '8.8.8.8',
    'secondary_dns': '1.1.1.1',
    'availability_zone': 'eu-de-01',
    'neutron_network_id': '4779ab1c-7c1a-44b1-a02e-93dfc361b32d',
    'neutron_subnet_id': '213cb9d-3122-2ac1-1a29-91ffc1231a12',
    'extra_dhcp_opts': [
        {
            'opt_value': '10.100.0.33,10.100.0.34',
            'opt_name': 'ntp'
        }
    ]
}


class TestVpc(base.TestCase):

    def setUp(self):
        super(TestVpc, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = subnet.Subnet()
        self.assertEqual('subnet', sot.resource_key)
        self.assertEqual('subnets', sot.resources_key)
        path = '/v1/%(project_id)s/subnets'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = subnet.Subnet(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(EXAMPLE['description'], sot.description)
        self.assertEqual(EXAMPLE['cidr'], sot.cidr)
        self.assertEqual(EXAMPLE['dnsList'], sot.dns_list)
        self.assertEqual(EXAMPLE['status'], sot.status)
        self.assertEqual(EXAMPLE['vpc_id'], sot.vpc_id)
        self.assertEqual(EXAMPLE['gateway_ip'], sot.gateway_ip)
        self.assertEqual(EXAMPLE['dhcp_enable'], sot.dhcp_enable)
        self.assertEqual(EXAMPLE['primary_dns'], sot.primary_dns)
        self.assertEqual(EXAMPLE['secondary_dns'], sot.secondary_dns)
        self.assertEqual(EXAMPLE['availability_zone'], sot.availability_zone)
        self.assertEqual(EXAMPLE['neutron_network_id'], sot.neutron_network_id)
        self.assertEqual(EXAMPLE['neutron_subnet_id'], sot.neutron_subnet_id)

        self.assertDictEqual(EXAMPLE['extra_dhcp_opts'][0],
                             sot.extra_dhcp_opts[0].to_dict(ignore_none=True))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import random
import uuid

# from openstack import resource

from otcextensions.tests.functional.sdk.waf import TestWaf


class TestDomain(TestWaf):

    _PRIVATE_KEY = """-----BEGIN PRIVATE KEY-----
MIIEvgIBADANBgkqhkiG9w0BAQEFAASCBKgwggSkAgEAAoIBAQDrvw+CfkRMtN6I
KQK+YNKhjdWqUCnTI7YqZLDhZkoIqcvK1F2mjkcoGXOAjCjvGXf/xX35j0dGLgHK
e3AwNaQDPRWec6DuTqh9kBq9Qy7rUs6Na85wwSN8FG7z9XRuWR9NhEg24nrATUr/
k5biBtHKiP22xI9nVws+IEoYVGOOjJ2CPt9XszaS/pdN6bQLchPSLOM5WN6BHCVn
u19RnSKFqCr8AmYx2Aqo30uFTHy1EhvSX8CnRTHNvWl7qciISqiNenXjIZyCe80n
7VB+LzSbm3HeqMDsM0euq0P5uPty2A6Uuo/TlPWlls6ZhFTP+AQ9H78kPWY9nKjl
1Ja4K0sbAgMBAAECggEANJ9oceOHkWvKRLCK2T45pjBH4oWUYHoXPq1NQnMX0Yk9
YWA4K2aVAaF0w9wFgyG3RJOsBBn0efjpE26sY0aF/ucSvVToNmm+eJDDNz4Y6hSI
4M6QvWCPcDILdk9zFvKz5xTBHec+KVDXjec/BeMpz0D3CWYk8JdgfhStFXM46eeR
z1KBOq51x+I0VD3Ar4T3hfKG2IViwevC/7kghBw+D1U/c4stHFCXv4JlrhFET2I6
kquGtV38fMUdWBLRVr0wBB4orm+9rpSlTvbnDuuEJcb8rKvrLkGraUhSTqepQD6M
lTN4BxY+3NqdnP/SKVBRoXr+gQsLdgPUAhkvTB8f0QKBgQD32mpyweaMZYTqZ8xF
xOBzjCTGVHNlXMMt8rz9+kJ4krJ77R3L07qf+mo5bsOB2ZybHhTy7+G6QO8TXyrI
60nbpoFR0eyWy6kdn4NtY/9BCcj13cV1D495zLr2HAveWDVVGJpLorkG5d674dtl
wD+B5EQIliCVR5GWMeciFGrewwKBgQDzfsU+EXlKAw6KMInyRP/+nWNk0PFir01H
Q4C/SrTM/Y8bCJ3/pWVAQsxEbQk1pOdWcdzHFf8BRncMA+OUDTxSCHJYiaqL+2pN
nNB3/bShocMKvDodJxXWMhdM2fMLFMtYCNsjr0DM8Cqvw7oZF8MY6oxM+uWzmI5R
nWKFMFXMyQKBgQDBK8PnKOSM69qJ7tgwUF827zUCNnOxvniIaTWPJOuFmZ/uIkIk
yCId6Ue892z82SPLacieBwQA6/bpPDTWXzszLDSCFoC0joqCAf6m1Vbt07iCl5P7
xmLmZQAaLIW7hzgZ2JD4/hwDGklcWY1rYkic7dFwd8FxV1RKoR4pW4xnjQKBgQDf
nEbU9kUVg/MhUuwL8fPJxo3VstBKWUS1sjcU9S1Op3h5UhOPBzwRpIZkPGHdwr+0
MkKXDgsuB6EiBpxDhVgk2Z7w0hQuE0gPWHhWCUaNvLkaLbuMtC0olL2zFOBPB9yp
zxA4GCSBT/lTioJnstu3EQahVzQFF49zQf6M49OXiQKBgCqOdwZjTH5gBnDSbWMM
WAFcxEzr5moG4nJzz/5sGqN5IRy1zDd/QkV2KEhjzWFbpGMgbgNTiLmz0BT6hUXl
/jS27B9AOPsdktyb88+ZuEfG6dYCmPnjBiOUrovbFk5IIAmiMAUT+W9HXN9shH0g
Ltxv392mcEGwmbfc1YJJfN2B
-----END PRIVATE KEY-----"""

    _CERTIFICATE = """-----BEGIN CERTIFICATE-----
MIIDADCCAegCCQCUu4mu6VfH/zANBgkqhkiG9w0BAQsFADBCMQswCQYDVQQGEwJE
RTELMAkGA1UEBwwCUEIxDDAKBgNVBAoMA1RTSTEYMBYGA1UEAwwPbXlmYWtlLnRl
c3QuY29tMB4XDTIwMDkwMTA5Mjc1M1oXDTIxMDkwMTA5Mjc1M1owQjELMAkGA1UE
BhMCREUxCzAJBgNVBAcMAlBCMQwwCgYDVQQKDANUU0kxGDAWBgNVBAMMD215ZmFr
ZS50ZXN0LmNvbTCCASIwDQYJKoZIhvcNAQEBBQADggEPADCCAQoCggEBAOu/D4J+
REy03ogpAr5g0qGN1apQKdMjtipksOFmSgipy8rUXaaORygZc4CMKO8Zd//FffmP
R0YuAcp7cDA1pAM9FZ5zoO5OqH2QGr1DLutSzo1rznDBI3wUbvP1dG5ZH02ESDbi
esBNSv+TluIG0cqI/bbEj2dXCz4gShhUY46MnYI+31ezNpL+l03ptAtyE9Is4zlY
3oEcJWe7X1GdIoWoKvwCZjHYCqjfS4VMfLUSG9JfwKdFMc29aXupyIhKqI16deMh
nIJ7zSftUH4vNJubcd6owOwzR66rQ/m4+3LYDpS6j9OU9aWWzpmEVM/4BD0fvyQ9
Zj2cqOXUlrgrSxsCAwEAATANBgkqhkiG9w0BAQsFAAOCAQEAcNsm1y3PgC47O7qW
3X531EiXmXsKuFWrpQeuVgSI/PrtXCn3/Gr1GcFQDA3k5iyDsApohwbyUcpXhA6c
842r2Flb11tMF7lxHwHGffryBeFbvCNSNYDvN9zA/XQfqpYi4UPPXPyLH0jVD0Ek
BCqJJFFzkRbUTcvTxCUxNEYpIQC8U4RSyWXg5kTu6302YjmWaNcP3bfL4II/ddI4
WyGW6tZI2z7GTYPutWljmtfgEto2Y3FimtiGU+P/uB6SxlESzkGEvAfEduGlyxY8
uslYHnizLvYY6FaAdExE1TpM6YrM3b7aYMgv700CDsBCpFncQUx9tujpQxCmMoHZ
rNcviQ==
-----END CERTIFICATE-----"""

    def setUp(self):
        super(TestDomain, self).setUp()

        self.cert_name = "SDK-D" + uuid.uuid4().hex
        self.domain_name = 'example-{0}.org'.format(random.randint(1, 10000))

        self.cert = self.client.create_certificate(
            key=self._PRIVATE_KEY,
            content=self._CERTIFICATE,
            name=self.cert_name
        )

        self.domain = self.client.create_domain(
            name=self.domain_name,
            certificate_id=self.cert.id,
            server=[dict(
                client_protocol="HTTPS",
                server_protocol="HTTP",
                address="1.2.3.4",
                port="80")],
            proxy=True,
            sip_header_name="default",
            sip_header_list=['X-Forwarded-For']
        )

        # reverse order is super important
        self.addCleanup(self.conn.waf.delete_certificate, self.cert)
        self.addCleanup(self.conn.waf.delete_domain, self.domain)

    def test_list_domains(self):
        cnt = 15
        # Pagination is so broken in WAF, that it makes sense to test it in
        # real, and not in units
        for i in range(0, cnt):
            domain = self.client.create_domain(
                name='%s.%s' % (i, self.domain_name),
                server=[dict(
                    client_protocol="HTTP",
                    server_protocol="HTTP",
                    address="1.2.3.4",
                    port="80")],
                proxy=False,
            )

            self.addCleanup(self.conn.waf.delete_domain, domain)

        query = {'limit': 3}
        domains = list(self.client.domains(**query))
        self.assertGreaterEqual(len(domains), cnt + 1)

        query = {'limit': 1}
        domains = list(self.client.domains(**query))
        self.assertGreaterEqual(len(domains), cnt + 1)

    def test_get_domain(self):
        domain = self.client.get_domain(self.domain.id)
        self.assertEqual(self.domain.name, domain.name)
        self.assertEqual(self.domain.id, domain.id)

    def test_find_domain(self):
        domain = self.client.find_domain(self.domain.name)
        self.assertEqual(self.domain.name, domain.name)
        self.assertEqual(self.domain.id, domain.id)

    def test_update_domain(self):
        cert2 = self.client.create_certificate(
            key=self._PRIVATE_KEY,
            content=self._CERTIFICATE,
            name=self.cert_name + "_2"
        )

        domain = self.client.update_domain(
            domain=self.domain,
            certificate_id=cert2.id,
        )
        self.assertEqual(domain.id, self.domain.id)
        self.addCleanup(self.conn.waf.delete_certificate, cert2)

        # We need to turn cert ref back, since otherwise cleanup can't drop
        # what is being used
        self.client.update_domain(
            domain=self.domain,
            certificate_id=self.cert.id,
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import json
import os

from keystoneauth1 import adapter

import mock

from openstack.tests.unit import base

from otcextensions.sdk.volume_backup.v2 import backup_policy as _backup_policy

TESTDATA_DIR = os.path.join(os.path.dirname(__file__))

EXAMPLE = {
    "id": "XX",
    "name": "plan01",
    "scheduled_policy": {
        "remain_first_backup_of_curMonth": 'N',
        "rentention_num": 10,
        "frequency": 1,
        "start_time": "12:00",
        "status": "ON"
    },
    "policy_resource_count": 0
}


def _get_fixture(name):
    fixture = os.path.join(
        TESTDATA_DIR,
        'data_files/',
        name)
    with open(fixture, 'r') as data:
        return json.load(data)


class TestBackupPolicy(base.TestCase):

    def setUp(self):
        super(TestBackupPolicy, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.post = mock.Mock()
        self.sess.put = mock.Mock()
        self.sess.delete = mock.Mock()

        self.sot = _backup_policy.BackupPolicy.existing(**EXAMPLE)

    def test_basic(self):
        sot = _backup_policy.BackupPolicy()
        self.assertEqual('backup_policies', sot.resources_key)
        self.assertEqual('/backuppolicy', sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_commit)

    def test_make_it(self):
        sot = _backup_policy.BackupPolicy.existing(**EXAMPLE)
        self.assertEqual(EXAMPLE['id'], sot.id)
        self.assertEqual(EXAMPLE['name'], sot.name)
        self.assertEqual(
            EXAMPLE['policy_resource_count'],
            sot.policy_resource_count)
        self.assertEqual(
            EXAMPLE['policy_resource_count'],
            sot.policy_resource_count)
        sp = sot.scheduled_policy
        sp_compare = EXAMPLE['scheduled_policy']
        assert isinstance(sp, _backup_policy.SchedulePolicy)
        self.assertEqual(
            sp_compare['remain_first_backup_of_curMonth'],
            sp.remain_first_backup_of_curMonth)
        self.assertEqual(
            sp_compare['rentention_num'],
            sp.rentention_num)
        self.assertEqual(
            sp_compare['frequency'],
            sp.frequency)
        self.assertEqual(
            sp_compare['start_time'],
            sp.start_time)

    def test_execute_policy(self):
        sot = _backup_policy.BackupPolicy.existing(id='XX')

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = {}

        self.sess.post.return_value = mock_response

        sot.execute(self.sess)

        self.sess.post.assert_called_once_with(
            'backuppolicy/XX/action',
            json=None
        )


class TestBackupPolicyResource(base.TestCase):

    def setUp(self):
        super(TestBackupPolicyResource, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.post = mock.Mock()

        self.sot = _backup_policy.BackupPolicyResource()

    def test_basic(self):
        sot = _backup_policy.BackupPolicyResource()
        self.assertEqual('/backuppolicyresources', sot.base_path)
        self.assertFalse(sot.allow_list)
        self.assertTrue(sot.allow_create)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)

    def test_make_it(self):
        sot = _backup_policy.BackupPolicyResource()

        self.assertIsNotNone(sot)
        assert isinstance(sot, _backup_policy.BackupPolicyResource)

    def test_link(self):
        response = _get_fixture('link_resources.json')

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = response

        self.sess.post.return_value = mock_response

        result = self.sot.link(
            self.sess, 'pol_id', ["volume-id-1", "volume-id-2"])

        self.sess.post.assert_called_once_with(
            '/backuppolicyresources',
            json={
                'resources': [
                    {'resource_id': 'volume-id-1', 'resource_type': 'volume'},
                    {'resource_id': 'volume-id-2', 'resource_type': 'volume'}
                ],
                'backup_policy_id': 'pol_id'
            }
        )

        self.assertEqual(1, len(result.success_resources))
        self.assertEqual(1, len(result.fail_resources))

    def test_unlink(self):
        response = _get_fixture('unlink_resources.json')

        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.headers = {}
        mock_response.json.return_value = response

        self.sess.post.return_value = mock_response

        result = self.sot.unlink(
            self.sess, 'pol_id', ["volume-id-1", "volume-id-2"])

        self.sess.post.assert_called_once_with(
            'backuppolicyresources/pol_id/deleted_resources',
            json={
                'resources': [
                    {'resource_id': 'volume-id-1', 'resource_type': 'volume'},
                    {'resource_id': 'volume-id-2', 'resource_type': 'volume'}
                ],
            }
        )

        self.assertEqual(1, len(result.success_resources))
        self.assertEqual(1, len(result.fail_resources))



#!/usr/bin/env python3
# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Create a service from attributes."""
import openstack

openstack.enable_logging(True)
conn = openstack.connect(cloud="otc")

attrs = {
    "service_name": "mnist",
    "description": "mnist service",
    "infer_type": "real-time",
    "config": [
        {
            "model_id": "xxxmodel-idxxx",
            "weight": "70",
            "specification": "modelarts.vm.cpu.2u",
            "instance_count": 1,
            "envs": {
                "model_name": "mxnet-model-1",
                "load_epoch": "0",
            },
        },
        {
            "model_id": "xxxxxx",
            "weight": "30",
            "specification": "modelarts.vm.cpu.2u",
            "instance_count": 1,
        },
    ],
}

service = conn.modelartsv1.create_service(**attrs)
print(service)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack.tests.unit import test_proxy_base
from otcextensions.sdk.modelartsv1.v1 import _proxy
from otcextensions.sdk.modelartsv1.v1 import builtin_model
from otcextensions.sdk.modelartsv1.v1 import devenv
from otcextensions.sdk.modelartsv1.v1 import job_engine
from otcextensions.sdk.modelartsv1.v1 import job_flavor
from otcextensions.sdk.modelartsv1.v1 import model
from otcextensions.sdk.modelartsv1.v1 import service
from otcextensions.sdk.modelartsv1.v1 import service_cluster
from otcextensions.sdk.modelartsv1.v1 import service_event
from otcextensions.sdk.modelartsv1.v1 import service_flavor
from otcextensions.sdk.modelartsv1.v1 import service_log
from otcextensions.sdk.modelartsv1.v1 import service_monitor
from otcextensions.sdk.modelartsv1.v1 import training_job
from otcextensions.sdk.modelartsv1.v1 import training_job_config
from otcextensions.sdk.modelartsv1.v1 import training_job_version
from otcextensions.sdk.modelartsv1.v1 import visualization_job


class TestModelartsV1Proxy(test_proxy_base.TestProxyBase):
    def setUp(self):
        super(TestModelartsV1Proxy, self).setUp()
        self.proxy = _proxy.Proxy(self.session)


class TestBuiltInModel(TestModelartsV1Proxy):
    def test_builtin_models(self):
        self.verify_list(
            self.proxy.builtin_models,
            builtin_model.BuiltInModel,
        )


class TestModel(TestModelartsV1Proxy):
    def test_models(self):
        self.verify_list(
            self.proxy.models,
            model.Model,
            expected_kwargs={"paginated": False},
        )

    def test_get_model(self):
        self.verify_get(
            self.proxy.get_model,
            model.Model,
        )

    def test_find_model(self):
        self.verify_find(
            self.proxy.find_model,
            model.Model,
            False,
        )

    def test_find_model_ignore(self):
        self.verify_find(
            self.proxy.find_model,
            model.Model,
            True,
        )

    def test_create_model(self):
        self.verify_create(
            self.proxy.create_model,
            model.Model,
            method_kwargs={"a": "b"},
            expected_kwargs={"prepend_key": False, "a": "b"},
        )

    def test_delete_model(self):
        self.verify_delete(
            self.proxy.delete_model,
            model.Model,
            False,
        )

    def test_delete_model_ignore(self):
        self.verify_delete(
            self.proxy.delete_devenv_instance,
            devenv.Devenv,
            True,
        )


class TestDevenv(TestModelartsV1Proxy):
    def test_devenv_instances(self):
        self.verify_list(
            self.proxy.devenv_instances,
            devenv.Devenv,
            method_kwargs={"limit": 10},
            expected_kwargs={
                "dev_env_type": "Notebook",
                "limit": 10,
                "paginated": False,
            },
        )

    def test_get_devenv_instance(self):
        self.verify_get(
            self.proxy.get_devenv_instance,
            devenv.Devenv,
        )

    def test_find_devenv_instance(self):
        self.verify_find(
            self.proxy.find_devenv_instance,
            devenv.Devenv,
            False,
            expected_kwargs={"de_type": "Notebook"},
        )

    def test_find_devenv_instance_ignore(self):
        self.verify_find(
            self.proxy.find_devenv_instance,
            devenv.Devenv,
            True,
            expected_kwargs={"de_type": "Notebook"},
        )

    def test_create_devenv_instance(self):
        self.verify_create(
            self.proxy.create_devenv_instance,
            devenv.Devenv,
            method_kwargs={"a": "b"},
            expected_kwargs={"prepend_key": False, "a": "b"},
        )

    def test_delete_devenv_instance(self):
        self.verify_delete(
            self.proxy.delete_devenv_instance,
            devenv.Devenv,
            False,
        )

    def test_delete_devenv_instance_ignore(self):
        self.verify_delete(
            self.proxy.delete_devenv_instance,
            devenv.Devenv,
            True,
        )

    def test_update_devenv_instance(self):
        self.verify_update(
            self.proxy.update_devenv_instance,
            devenv.Devenv,
        )

    def test_start_devenv_instance(self):
        self._verify(
            "otcextensions.sdk.modelartsv1.v1.devenv.Devenv.start",
            self.proxy.start_devenv_instance,
            method_args=["val"],
            expected_args=[self.proxy],
        )

    def test_stop_devenv_instance(self):
        self._verify(
            "otcextensions.sdk.modelartsv1.v1.devenv.Devenv.stop",
            self.proxy.stop_devenv_instance,
            method_args=["val"],
            expected_args=[self.proxy],
        )


class TestService(TestModelartsV1Proxy):
    def test_services(self):
        self.verify_list(
            self.proxy.services,
            service.Service,
            method_kwargs={"limit": 10},
            expected_kwargs={"limit": 10, "paginated": False},
        )

    def test_get_service(self):
        self.verify_get(
            self.proxy.get_service,
            service.Service,
        )

    def test_create_service(self):
        self.verify_create(
            self.proxy.create_service,
            service.Service,
            method_kwargs={"a": "b"},
            expected_kwargs={"a": "b"},
        )

    def test_start_service(self):
        self.verify_update(
            self.proxy.update_service,
            service.Service,
            method_kwargs={"status": "running"},
            expected_kwargs={"status": "running"},
        )

    def test_stop_service(self):
        self.verify_update(
            self.proxy.update_service,
            service.Service,
            method_kwargs={"status": "stopped"},
            expected_kwargs={"status": "stopped"},
        )

    def test_delete_service(self):
        self.verify_delete(
            self.proxy.delete_service,
            service.Service,
            False,
        )

    def test_delete_service_ignore(self):
        self.verify_delete(
            self.proxy.delete_service,
            service.Service,
            True,
        )

    def test_update_service(self):
        self.verify_update(
            self.proxy.update_service,
            service.Service,
        )


class TestJobFlavor(TestModelartsV1Proxy):
    def test_job_flavors(self):
        self.verify_list(
            self.proxy.job_flavors,
            job_flavor.JobFlavor,
        )


class TestJobEngine(TestModelartsV1Proxy):
    def test_job_engines(self):
        self.verify_list(
            self.proxy.job_engines,
            job_engine.JobEngine,
        )


class TestServiceLog(TestModelartsV1Proxy):
    def test_service_logs(self):
        self.verify_list(
            self.proxy.service_logs,
            service_log.ServiceLog,
            method_kwargs={"service": "service-id"},
            expected_kwargs={"service_id": "service-id"},
        )


class TestServiceEvent(TestModelartsV1Proxy):
    def test_service_events(self):
        self.verify_list(
            self.proxy.service_events,
            service_event.ServiceEvent,
            method_kwargs={"service": "service-id"},
            expected_kwargs={"service_id": "service-id"},
        )


class TestServiceMonitor(TestModelartsV1Proxy):
    def test_service_monitor(self):
        self.verify_list(
            self.proxy.service_monitor,
            service_monitor.ServiceMonitor,
            method_kwargs={"service": "service-id"},
            expected_kwargs={"service_id": "service-id"},
        )


class TestServiceFlavor(TestModelartsV1Proxy):
    def test_service_flavors(self):
        self.verify_list(
            self.proxy.service_flavors,
            service_flavor.ServiceFlavor,
        )


class TestServiceCluster(TestModelartsV1Proxy):
    def test_service_clusters(self):
        self.verify_list(
            self.proxy.service_clusters,
            service_cluster.ServiceCluster,
        )


class TestTrainingJob(TestModelartsV1Proxy):
    def test_training_jobs(self):
        self.verify_list(
            self.proxy.training_jobs,
            training_job.TrainingJob,
        )

    def test_create_training_job(self):
        self.verify_create(
            self.proxy.create_training_job,
            training_job.TrainingJob,
        )

    def test_update_training_job(self):
        trainingjob_instance = training_job.TrainingJob()
        description = "test description"
        self._verify(
            "openstack.proxy.Proxy._update",
            self.proxy.update_training_job,
            method_args=[trainingjob_instance, description],
            expected_args=[training_job.TrainingJob, trainingjob_instance],
            expected_kwargs={"job_desc": description},
        )

    def test_delete_training_job(self):
        self.verify_delete(
            self.proxy.delete_training_job,
            training_job.TrainingJob,
            False,
        )

    def test_delete_training_job_ignore(self):
        self.verify_delete(
            self.proxy.delete_training_job,
            training_job.TrainingJob,
            True,
        )


class TestTrainingJobVersion(TestModelartsV1Proxy):
    def test_training_job_versions(self):
        self.verify_list(
            self.proxy.training_job_versions,
            training_job_version.TrainingJobVersion,
            method_kwargs={"training_job": "job-id"},
            expected_kwargs={"job_id": "job-id"},
        )

    def test_create_training_job_version(self):
        self.verify_create(
            self.proxy.create_training_job_version,
            training_job_version.TrainingJobVersion,
            method_kwargs={"training_job": "job-id"},
            expected_kwargs={"job_id": "job-id"},
        )

    def test_get_training_job_version(self):
        self.verify_get(
            self.proxy.get_training_job_version,
            training_job_version.TrainingJobVersion,
            method_args=["job-id", "version-id"],
            expected_args=["version-id"],
            expected_kwargs={"job_id": "job-id"},
        )

    def test_delete_training_job_version(self):
        self.verify_delete(
            self.proxy.delete_training_job_version,
            training_job_version.TrainingJobVersion,
            False,
            method_args=["job-id", "version-id"],
            expected_args=["version-id"],
            expected_kwargs={"job_id": "job-id"},
        )

    def test_delete_training_job_version_ignore(self):
        self.verify_delete(
            self.proxy.delete_training_job_version,
            training_job_version.TrainingJobVersion,
            True,
            method_args=["job-id", "version-id"],
            expected_args=["version-id"],
            expected_kwargs={"job_id": "job-id"},
        )

    def test_stop_training_job_version(self):
        self._verify(
            "otcextensions.sdk.modelartsv1.v1.training_job_version.TrainingJobVersion.stop",  # noqa: E501
            self.proxy.stop_training_job_version,
            method_args=["job-id", "version-id"],
            expected_args=[self.proxy],
        )


class TestTrainingJobConfig(TestModelartsV1Proxy):
    def test_training_job_configs(self):
        self.verify_list(
            self.proxy.training_job_configs,
            training_job_config.TrainingJobConfig,
        )

    def test_create_training_job_config(self):
        self.verify_create(
            self.proxy.create_training_job_config,
            training_job_config.TrainingJobConfig,
        )

    def test_get_training_job_config(self):
        self.verify_get(
            self.proxy.get_training_job_config,
            training_job_config.TrainingJobConfig,
        )

    def test_update_training_job_config(self):
        self.verify_update(
            self.proxy.update_training_job_config,
            training_job_config.TrainingJobConfig,
        )

    def test_delete_training_job_config(self):
        self.verify_delete(
            self.proxy.delete_training_job_config,
            training_job_config.TrainingJobConfig,
            False,
        )

    def test_delete_training_job_config_ignore(self):
        self.verify_delete(
            self.proxy.delete_training_job_config,
            training_job_config.TrainingJobConfig,
            True,
        )


class TestVisualizationJob(TestModelartsV1Proxy):
    def test_visualization_jobs(self):
        self.verify_list(
            self.proxy.visualization_jobs,
            visualization_job.VisualizationJob,
        )

    def test_create_visualization_job(self):
        self.verify_create(
            self.proxy.create_visualization_job,
            visualization_job.VisualizationJob,
        )

    def test_update_visualization_job(self):
        visualizationjob_instance = visualization_job.VisualizationJob()
        description = "test description"
        self._verify(
            "openstack.proxy.Proxy._update",
            self.proxy.update_visualization_job,
            method_args=[visualizationjob_instance, description],
            expected_args=[
                visualization_job.VisualizationJob,
                visualizationjob_instance,
            ],
            expected_kwargs={"job_desc": description},
        )

    def test_delete_visualization_job(self):
        self.verify_delete(
            self.proxy.delete_visualization_job,
            visualization_job.VisualizationJob,
            False,
        )

    def test_delete_visualization_job_ignore(self):
        self.verify_delete(
            self.proxy.delete_visualization_job,
            visualization_job.VisualizationJob,
            True,
        )

    def test_restart_visualization_job(self):
        self._verify(
            (
                "otcextensions.sdk.modelartsv1.v1.visualization_job."
                "VisualizationJob.restart"
            ),
            self.proxy.restart_visualization_job,
            method_args=["val"],
            expected_args=[self.proxy],
        )

    def test_stop_visualization_job(self):
        self._verify(
            "otcextensions.sdk.modelartsv1.v1.visualization_job.VisualizationJob.stop",  # noqa: E501
            self.proxy.stop_visualization_job,
            method_args=["val"],
            expected_args=[self.proxy],
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack.tests.unit import base
from otcextensions.sdk.modelartsv1.v1 import service_log
from otcextensions.tests.unit.sdk.modelartsv1.v1 import examples
from otcextensions.tests.unit.sdk.utils import assert_attributes_equal

EXAMPLE = examples.SERVICE_LOG


class TestLog(base.TestCase):
    def setUp(self):
        super(TestLog, self).setUp()

    def test_basic(self):
        sot = service_log.ServiceLog()

        self.assertEqual("/services/%(service_id)s/logs", sot.base_path)
        self.assertEqual("logs", sot.resources_key)
        self.assertEqual(None, sot.resource_key)

        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)
        self.assertDictEqual(
            {
                "update_time": "update_time",
                "limit": "limit",
                "marker": "marker",
            },
            sot._query_mapping._mapping,
        )

    def test_make_it(self):
        sot = service_log.ServiceLog(**EXAMPLE)
        assert_attributes_equal(self, sot, EXAMPLE)



#   Licensed under the Apache License, Version 2.0 (the "License"); you may
#   not use this file except in compliance with the License. You may obtain
#   a copy of the License at
#
#        http://www.apache.org/licenses/LICENSE-2.0
#
#   Unless required by applicable law or agreed to in writing, software
#   distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
#   WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
#   License for the specific language governing permissions and limitations
#   under the License.
#
'''CMK v1 action implementations'''
import logging

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.command import command

from otcextensions.i18n import _

LOG = logging.getLogger(__name__)


def timestamp_to_str(obj):
    """Convert KMS timestamp to Humanreadable format
    """
    # TODO(anybody) Convert to string when clear how
    return obj


class ListCMK(command.Lister):
    _description = _('List Customer Master Keys (CMK)')
    columns = ('ID', 'key_alias', 'key_state')

    def get_parser(self, prog_name):
        parser = super(ListCMK, self).get_parser(prog_name)
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            default=None,
            help=_('Limit the number of results fetch at a time')
        )
        parser.add_argument(
            '--state',
            metavar='<state>',
            type=int,
            choices=range(0, 5),
            help=_('CMK state:\n'
                   '`1` - waiting for activation\n'
                   '`2` - enabled\n'
                   '`3` - disabled\n'
                   '`4` - scheduled for deletion')
        )

        return parser

    def take_action(self, parsed_args):

        args = {}
        if parsed_args.limit:
            args['limit'] = parsed_args.limit
        if parsed_args.state:
            args['key_state'] = parsed_args.state

        client = self.app.client_manager.kms

        data = client.keys(**args)

        return (
            self.columns,
            (utils.get_item_properties(
                s,
                self.columns,
            ) for s in data)
        )


class ShowCMK(command.ShowOne):
    _description = _('Shows details of a CMK')
    columns = ['ID', 'key_alias', 'domain_id', 'realm',
               'key_description', 'creation_date', 'scheduled_deletion_date',
               'key_state', 'key_type']

    def get_parser(self, prog_name):
        parser = super(ShowCMK, self).get_parser(prog_name)
        parser.add_argument(
            'key',
            metavar='<key>',
            help=_('ID or the alias of the CMK')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.kms

        obj = None

        if parsed_args.key:
            obj = client.find_key(parsed_args.key)

        data = utils.get_item_properties(
            obj, self.columns, formatters={})

        return (self.columns, data)


class EnableCMK(command.Command):
    _description = _('Enables the CMK')

    def get_parser(self, prog_name):
        parser = super(EnableCMK, self).get_parser(prog_name)
        parser.add_argument(
            'key',
            metavar='<key>',
            help=_('ID or the alias of the CMK')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.kms

        obj = None

        if parsed_args.key:
            obj = client.find_key(parsed_args.key)

        if obj:
            client.enable_key(obj)

        return None


class DisableCMK(command.Command):
    _description = _('Disables the CMK')

    def get_parser(self, prog_name):
        parser = super(DisableCMK, self).get_parser(prog_name)
        parser.add_argument(
            'key',
            metavar='<key>',
            help=_('ID or the alias of the CMK')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.kms

        obj = None

        if parsed_args.key:
            obj = client.find_key(parsed_args.key)

        if obj:
            client.disable_key(obj)

        return None


class DeleteCMK(command.Command):
    _description = _('Schedules deletion of the CMK')

    def get_parser(self, prog_name):
        parser = super(DeleteCMK, self).get_parser(prog_name)
        parser.add_argument(
            'key',
            metavar='<key>',
            help=_('ID or the alias of the CMK')
        )
        parser.add_argument(
            'days',
            metavar='<days>',
            type=int,
            help=_('Number of days in future after which CMK '
                   'will be deleted [7..1096]')
        )

        return parser

    def take_action(self, parsed_args):

        obj = None
        days = 7

        if parsed_args.days:
            days = parsed_args.days
            if not 7 <= days <= 1096:
                msg = _('Invalid number of days. '
                        'Please choose in range (7..1096)')
                raise exceptions.CommandError(msg)

        client = self.app.client_manager.kms

        if parsed_args.key:
            obj = client.find_key(parsed_args.key)

        if obj:
            client.schedule_key_deletion(key=obj, pending_days=days)

        return None


class CancelDeleteCMK(command.Command):
    _description = _('Cancels the scheduled deletion of the CMK')

    def get_parser(self, prog_name):
        parser = super(CancelDeleteCMK, self).get_parser(prog_name)
        parser.add_argument(
            'key',
            metavar='<key>',
            help=_('ID or the alias of the CMK')
        )

        return parser

    def take_action(self, parsed_args):

        obj = None

        client = self.app.client_manager.kms

        if parsed_args.key:
            obj = client.find_key(parsed_args.key)

        if obj:
            client.cancel_key_deletion(key=obj)


class CreateCMK(command.ShowOne):
    _description = _('Creates CMK')
    columns = ['ID', 'key_alias', 'domain_id', 'realm',
               'key_description', 'creation_date', 'scheduled_deletion_date',
               'key_state', 'key_type']
    POLICY_TYPES = ['ALARM', 'SCHEDULED', 'RECURRENCE']

    def get_parser(self, prog_name):
        parser = super(CreateCMK, self).get_parser(prog_name)
        parser.add_argument(
            'alias',
            metavar='<alias>',
            help=_('CMK Alias')
        )
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('CMK description')
        )
        parser.add_argument(
            '--realm',
            metavar='<realm>',
            help=_('Realm value')
        )
        parser.add_argument(
            '--key_policy',
            metavar='<key_policy>',
            help=_('Specifies the key policy')
        )
        parser.add_argument(
            '--key_usage',
            metavar='<key_usage>',
            help=_('Purpose of the CMK')
        )
        parser.add_argument(
            '--type',
            metavar='<type>',
            help=_('Type of the CMK')
        )
        return parser

    def take_action(self, parsed_args):

        attrs = {}
        attrs['key_alias'] = parsed_args.alias
        if parsed_args.description:
            attrs['key_description'] = parsed_args.description
        if parsed_args.realm:
            attrs['realm'] = parsed_args.realm
        if parsed_args.key_policy:
            attrs['key_policy'] = parsed_args.key_policy
        if parsed_args.key_usage:
            attrs['key_usage'] = parsed_args.key_usage
        if parsed_args.type:
            attrs['key_type'] = parsed_args.type

        client = self.app.client_manager.kms

        cmk = client.create_key(**attrs)

        data = utils.get_item_properties(
            cmk, self.columns, formatters={})

        return (self.columns, data)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import proxy

from otcextensions.common.utils import extract_url_parts
from otcextensions.sdk.swr.v2 import _base
from otcextensions.sdk.swr.v2 import organization as _organization
from otcextensions.sdk.swr.v2 import repository as _repository
from otcextensions.sdk.swr.v2 import domain as _domain


def update_permissions_attrs(attrs):
    for permission in attrs.get('permissions', []):
        permission['auth'] = permission.pop('user_auth', None)
    return attrs


class Proxy(proxy.Proxy):
    skip_discovery = True

    def _extract_name(self, url, service_type=None, project_id=None):
        return extract_url_parts(url, project_id)

    def create_organization(self, **attrs):
        """Create a new organization from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.swr.v2.
            organization.Organization`, comprised of the properties on the
            Organization class.

        :returns: The results of organization creation
        :rtype: :class:`~otcextensions.sdk.swr.v2.organization.Organization`
        """
        return self._create(_organization.Organization, **attrs)

    def get_organization(self, *attrs):
        """Get an organization

        :returns: One
             :class:`~otcextensions.sdk.swr.v2.organization.Organization`
        """
        return self._get(_organization.Organization, *attrs)

    def organizations(self, **query):
        """Retrieve a generator of organizations

        :returns: A generator of organization instances
        """
        return self._list(_organization.Organization, **query)

    def delete_organization(self, namespace, ignore_missing=True):
        """Delete an organization

        :param namespace: The namespace can be either the name or a
            :class:`~otcextensions.sdk.swr.v2.organization.Organization`
            instance
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the load balancer does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent organization.

        :returns: ``None``
        """
        orgobj = self._get_resource(_organization.Organization, namespace)
        return self._delete(_organization.Organization, orgobj.id,
                            ignore_missing=ignore_missing)

    def find_organization(self, name_or_id, ignore_missing=True):
        """Find a single organization

        :param name_or_id: The name or ID of an organization
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised
            when the organization does not exist.
            When set to ``True``, no exception will be set when attempting
            to delete a nonexistent organization.

        :returns: ``None``
        """
        return self._find(_organization.Organization, name_or_id,
                          ignore_missing=ignore_missing)

    def create_organization_permissions(self, **attrs):
        """Create a new organization from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.swr.v2.
            organization.Organization`, comprised of the properties on the
            Organization class.

        :returns: The results of organization creation
        :rtype: :class:`~otcextensions.sdk.swr.v2.organization.Permission`
        """
        attrs = update_permissions_attrs(attrs)
        return self._create(_organization.Permission, **attrs)

    def organization_permissions(self, namespace, **query):
        """Retrieve a generator of organization permissions

        :returns: A generator of organization permissions instances
        """
        return self._list(_organization.Permission,
                          namespace=namespace, **query)

    def delete_organization_permissions(
            self, namespace, user_ids, ignore_missing=True
    ):
        """Delete an organization permissions

        :param user_ids: Users IDs whose permissions need to be deleted.
        :param namespace: The namespace can be either the name or a
            :class:`~otcextensions.sdk.swr.v2.organization.Permission`
            instance
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the load balancer does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent permission.

        :returns: ``None``
        """
        base_path = f'/manage/namespaces/{namespace}/access'
        res = self._get_resource(_base.Resource, namespace)
        return res._delete(self, user_ids, base_path)

    def update_organization_permissions(self, **attrs):
        """Update an organization permissions

        :param dict attrs: The attributes to update on the permissions
         represented by ``permissions``.

        :returns: The updated permissions.

        :rtype: :class:`~otcextensions.sdk.swr.v2.organization.Permission`
        """
        attrs = update_permissions_attrs(attrs)
        return self._update(_organization.Permission, **attrs)

    def create_repository(self, **attrs):
        """Create a new image repository from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.swr.v2.
            repository.Repository`, comprised of the properties on the
            Repository class.

        :returns: The results of organization creation
        :rtype: :class:`~otcextensions.sdk.swr.v2.repository.Repository`
        """
        return self._create(_repository.Repository, **attrs)

    def delete_repository(
            self, namespace, repository, ignore_missing=True
    ):
        """Delete a repository

        :param repository: Image repository name need to be deleted.
        :param namespace: The namespace can be either the name or a
            :class:`~otcextensions.sdk.swr.v2.repository.Repository`
            instance
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the load balancer does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent repository.

        :returns: ``None``
        """
        orgobj = self._get_resource(_organization.Organization, namespace)
        repotype = _repository.Repository
        repotype.requires_id = True
        self._delete(repotype, repository,
                     ignore_missing=ignore_missing, namespace=orgobj.id)

    def get_repository(self, namespace, repository):
        """Get a repository

        :returns: One
             :class:`~otcextensions.sdk.swr.v2.repository.Repository`
        """
        orgobj = self._get_resource(_organization.Organization, namespace)
        return self._get(_repository.Repository, repository,
                         namespace=orgobj.id)

    def repositories(self, **query):
        """Retrieve a generator of repositories

        :returns: A generator of repositories instances
        """
        base_path = '/manage/repos'
        return self._list(_repository.Repository, base_path=base_path, **query)

    def update_repository(self, **attrs):
        """Update a repository

        :param dict attrs: The attributes to update on the repository
         represented by ``repository``.

        :returns: The updated repository.

        :rtype: :class:`~otcextensions.sdk.swr.v2.repository.Repository`
        """
        bp = f'/manage/namespaces/%(namespace)s/repos/{attrs["repository"]}'
        repotype = _repository.Repository
        repotype.requires_id = False
        return self._update(repotype, "", base_path=bp, **attrs)

    def create_repository_permissions(self, **attrs):
        """Create a new repository permissions from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.swr.v2.
            repository.Permission`, comprised of the properties on the
            Permission class.

        :returns: The results of repository permission creation
        :rtype: :class:`~otcextensions.sdk.swr.v2.repository.Permission`
        """
        attrs = update_permissions_attrs(attrs)
        return self._create(_repository.Permission, **attrs)

    def repository_permissions(self, namespace, repository, **query):
        """Retrieve a generator of repository permissions

        :returns: A generator of repository permissions instances
        """
        return self._list(_repository.Permission,
                          namespace=namespace,
                          repository=repository,
                          **query)

    def delete_repository_permissions(
            self, namespace, repository, user_ids, ignore_missing=True
    ):
        """Delete a repository permissions

        :param repository: The repository can be either the name or a
            :class:`~otcextensions.sdk.swr.v2.repository.Permission`
            instance
        :param user_ids: Users IDs whose permissions need to be deleted.
        :param namespace: The namespace can be either the name or a
            :class:`~otcextensions.sdk.swr.v2.organization.Permission`
            instance
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the load balancer does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent permission.

        :returns: ``None``
        """
        bp = f'/manage/namespaces/{namespace}/repos/{repository}/access'
        res = self._get_resource(_base.Resource, namespace)
        return res._delete(self, user_ids, bp)

    def update_repository_permissions(self, **attrs):
        """Update a repository permissions

        :param dict attrs: The attributes to update on the permissions
         represented by ``permissions``.

        :returns: The updated permissions.

        :rtype: :class:`~otcextensions.sdk.swr.v2.repository.Permission`
        """
        attrs = update_permissions_attrs(attrs)
        return self._update(_repository.Permission, **attrs)

    def create_domain(self, **attrs):
        """Create a new image repository from attributes

        :param dict attrs: Keyword arguments which will be used to create
            a :class:`~otcextensions.sdk.swr.v2.
            domain.Domain`, comprised of the properties on the
            Repository class.

        :returns: The results of organization creation
        :rtype: :class:`~otcextensions.sdk.swr.v2.domain.Domain`
        """
        return self._create(_domain.Domain, **attrs)

    def delete_domain(
            self, namespace, repository, access_domain, ignore_missing=True
    ):
        """Delete a domain

        :param access_domain: Name of the account need to be deleted.
        :param repository: Image repository name.
        :param namespace: Organization name.
        :param bool ignore_missing: When set to ``False``
            :class:`~openstack.exceptions.ResourceNotFound` will be raised when
            the load balancer does not exist.
            When set to ``True``, no exception will be set when attempting to
            delete a nonexistent repository.

        :returns: ``None``
        """
        self._delete(_domain.Domain, repository=repository,
                     id=access_domain,
                     ignore_missing=ignore_missing,
                     namespace=namespace)

    def domains(self, **query):
        """Retrieve a generator of domains

        :returns: A generator of domain instances
        """
        return self._list(_domain.Domain, **query)

    def update_domain(self, **attrs):
        """Update a domain

        :param dict attrs: The attributes to update on the domain
         represented by ``permissions``.

        :returns: The updated domain.

        :rtype: :class:`~otcextensions.sdk.swr.v2.domain.Domain`
        """
        return self._update(_domain.Domain, id=attrs['access_domain'], **attrs)

    def get_domain(self, namespace, repository, access_domain):
        """Get a domain

        :returns: One
             :class:`~otcextensions.sdk.swr.v2.domain.Domain`
        """
        return self._get(_domain.Domain,
                         repository=repository,
                         id=access_domain,
                         namespace=namespace)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from unittest import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.dds.v3 import datastore


EXAMPLE = {
    "storage_engine": "wiredTiger",
    "type": "DDS-Community",
    "version": "3.2"}


class TestDatastore(base.TestCase):
    def setUp(self):
        super(TestDatastore, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.get = mock.Mock()
        self.sess.default_microversion = None
        self.sess._get_connection = mock.Mock(return_value=self.cloud)

    def test_basic(self):
        sot = datastore.Datastore()
        self.assertEqual(
            '/datastores',
            sot.base_path)
        self.assertTrue(sot.allow_list)
        self.assertFalse(sot.allow_fetch)
        self.assertFalse(sot.allow_create)
        self.assertFalse(sot.allow_delete)
        self.assertFalse(sot.allow_commit)
        self.assertDictEqual({
            'limit': 'limit',
            'marker': 'marker'
        }, sot._query_mapping._mapping)

    def test_make_it(self):
        sot = datastore.Datastore(**EXAMPLE)
        self.assertEqual(EXAMPLE['storage_engine'], sot.storage_engine)
        self.assertEqual(EXAMPLE['type'], sot.type)
        self.assertEqual(EXAMPLE['version'], sot.version)

    def test_list(self):
        sot = datastore.Datastore()
        mock_response = mock.Mock()
        mock_response.status_code = 200
        mock_response.json.return_value = {"versions": ["1", "2"]}
        self.sess.get.return_value = mock_response

        datastore_name = 'foo'
        res = list(sot.list(self.sess, datastore_name=datastore_name))
        self.sess.get.assert_called_with(
            f'datastores/{datastore_name}/versions',
            headers={'Accept': 'application/json'},
            microversion=None
        )
        self.assertEqual(2, len(res))
        self.assertEqual(
            datastore.Datastore(
                type=datastore_name,
                storage_engine='wiredTiger',
                version='1'
            ).to_dict(computed=False),
            res[0].to_dict(computed=False)
        )



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
from openstack import resource
from openstack import utils
from otcextensions.sdk.modelartsv1.v1.training_job import ConfigSpec


class TrainingJobVersion(ConfigSpec):
    base_path = "/training-jobs/%(job_id)s/versions"

    resources_key = "versions"

    # capabilities
    allow_create = True
    allow_delete = True
    allow_fetch = True
    allow_list = True

    _query_mapping = resource.QueryParameters(
        "limit",
        "offset",
        limit="per_page",
        offset="page",
    )

    # Properties
    #: Job ID
    job_id = resource.URI("job_id")
    #: Parameters for creating a training job For details, see Table 3.
    config = resource.Body("config", type=ConfigSpec)
    #: Description of a training job.
    job_description = resource.Body("job_desc")
    #: Version ID of a training job.
    version_id = resource.Body("version_id", type=int, alternate_id=True)

    def stop(self, session):
        """Preform actions given the message body."""
        uri = utils.urljoin(
            self.base_path % self._uri.attributes, self.id, "stop"
        )
        response = session.post(uri, json=None)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import _log
from openstack import resource
from openstack import utils
from openstack import exceptions

LOG = _log.setup_logging(__name__)


class Share(resource.Resource):

    base_path = '/sfs-turbo/shares'
    resource_key = 'share'
    resources_key = 'shares'

    # capabilities
    allow_create = True
    allow_fetch = True
    allow_delete = True
    allow_list = True

    #: Specifies the creation progress of the SFS Turbo file system.
    #: *Type: dict*
    action_progress = resource.Body('action_progress', type=dict)
    #: Specifies the name of the AZ where the SFS Turbo file system is located.
    #: *Type: str*
    az_name = resource.Body('az_name')
    #: Specifies the available capacity of the SFS Turbo file system in the
    # unit of GB.
    #: *Type: str*
    avail_capacity = resource.Body('avail_capacity')
    #: Specifies the code of the AZ where the SFS Turbo file system is located.
    #: *Type: str*
    availability_zone = resource.Body('availability_zone')
    #: Specifies the creation time.
    #: *Type: str*
    created_at = resource.Body('created_at')
    #: Specifies the ID of the encryption key specified by the user.
    # This parameter is not returned for non-encrypted disks.
    #: *Type: str*
    crypt_key_id = resource.Body('crypt_key_id')
    #: For an enhanced file system, bandwidth is returned for this field.
    #: *Type: str*
    expand_type = resource.Body('expand_type')
    #: Specifies the mount point of the SFS Turbo file system.
    #: *Type: str*
    export_location = resource.Body('export_location')
    #: Specifies the name of the SFS Turbo file system.
    #: *Type: str*
    name = resource.Body('name')
    #: Specifies the status of the SFS Turbo file system.
    #: *Type: str*
    status = resource.Body('status')
    #: Specifies the sub-status of the SFS Turbo file system.
    #: *Type:str*
    sub_status = resource.Body('sub_status')
    #: Specifies the type of the SFS Turbo file system.
    #: *Type: str*
    share_type = resource.Body('share_type')
    #: Specifies the region of the SFS Turbo file system.
    #: *Type: str*
    region = resource.Body('region')
    #: Specifies the network ID of the subnet specified by the user.
    #: *Type: str*
    subnet_id = resource.Body('subnet_id')
    #: Specifies the ID of a security group specified by the user.
    #: *Type: str*
    security_group_id = resource.Body('security_group_id')
    #: Specifies the total capacity of the SFS Turbo file system in the
    # unit of GB.
    #: *Type: str*
    size = resource.Body('size')
    #: Specifies the protocol type of the SFS Turbo file system.
    # The current value is NFS.
    #: *Type: str*
    share_proto = resource.Body('share_proto')
    #: Billing mode of the SFS Turbo file system.
    #: *Type: str*
    pay_model = resource.Body('pay_model')
    #: Project id.
    #: *Type: str*
    project_id = resource.URI('project_id')
    #: Specifies the version ID of the SFS Turbo file system.
    #: *Type:str*
    version = resource.Body('version')
    #: Specifies the VPC ID specified by the user.
    #: *Type: str*
    vpc_id = resource.Body('vpc_id')

    def extend_capacity(self, session, extend):
        """Method to extend the capacity of the file system

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param dict extend: Specifies the extend object.
        """
        url = utils.urljoin(self.base_path, self.id, 'action')
        body = {
            'extend': extend
        }
        response = session.post(url, json=body)
        return self._to_object(session, response)

    def change_security_group(self, session, change_security_group):
        """Method to change the security group bound to the file system.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param dict change_security_group: Specifies the change_security_group
            object.
        """
        url = utils.urljoin(self.base_path, self.id, 'action')
        body = {
            'change_security_group': change_security_group
        }
        response = session.post(url, json=body)
        return self._to_object(session, response)

    def _to_object(self, session, response):
        has_body = (
            self.has_body
            if self.create_returns_body is None
            else self.create_returns_body
        )
        microversion = self._get_microversion(session)
        self.microversion = microversion
        self._translate_response(response, has_body=has_body)
        if self.has_body and self.create_returns_body is False:
            return self.fetch(session)
        return self

    def wait_for_substatus(
        self,
        session,
        desired_substatus,
        interval=5,
        wait=350,
        failure=None,
        callback=None,
    ):
        """Wait for sub_status to reach desired state.

        :param session: The session to use for making this request.
        :type session: :class:`~keystoneauth1.adapter.Adapter`
        :param int interval: Number of seconds to wait between checks.
            Set to ``None`` to use the default interval.
        :param int wait: Maximum number of seconds to wait for transition.
            Set to ``None`` to wait forever.
        :param list failure: Sub status which means failure.
        :param callback: A callback function. This will be called with a single
            value, progress. This is API specific but is generally a percentage
            value from 0-100.

        :return: The share object.
        :raises: :class:`~openstack.exceptions.ResourceTimeout` transition
            to status failed to occur in wait seconds.
        :raises: :class:`~openstack.exceptions.ResourceFailure` if the resource
                 has transited to one of the failure statuses.
        """

        resource = self.fetch(session, skip_cache=True)
        current_substatus = getattr(resource, 'sub_status')
        if current_substatus == desired_substatus:
            return self

        name = "{res}:{id}".format(res=self.__class__.__name__,
                                   id=self.id)
        msg = "Timeout waiting for {name} to extend capacity".\
            format(name=name)

        for count in utils.iterate_timeout(
            timeout=wait, message=msg, wait=interval
        ):
            resource = self.fetch(session, skip_cache=True)

            current_substatus = getattr(resource, 'sub_status')
            if current_substatus == desired_substatus:
                return resource
            elif current_substatus == failure:
                raise exceptions.ResourceFailure(
                    "{name} transitioned to failure state {status}".format(
                        name=name, status=failure
                    )
                )

            LOG.debug(
                'Still waiting for resource %s to extend capacity',
                name
            )

            if callback:
                progress = getattr(resource, 'progress', None) or 0
                callback(progress)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource

from otcextensions.sdk.cce.v3 import _base


class HostNetworkSpec(resource.Resource):

    # Properties
    #: ID of the high-speed network that is used to create a bare metal node.
    highway_subnet = resource.Body('highwaySubnet')
    #: Security group.
    security_group = resource.Body('SecurityGroup')
    #: ID of the subnet that is used to create a node.
    network_id = resource.Body('subnet')
    #: ID of the VPC that is used to create a node.
    router_id = resource.Body('vpc')


class EniNetworkSpec(resource.Resource):

    # Properties
    #: IPv4 Subnet ID of the ENI container subnet.
    eni_subnet_id = resource.Body('eniSubnetId')
    #: ENI subnet CIDR block.
    eni_subnet_cidr = resource.Body('eniSubnetCIDR')


class ClusterSpec(resource.Resource):

    #: Authentication
    authentication = resource.Body('authentication', type=dict)
    #: Billing mode of the cluster. Currently, only pay-per-use is supported.
    billing = resource.Body('billingMode')
    #: Container network parameters.
    container_network = resource.Body('containerNetwork', type=dict)
    #: Cluster description.
    description = resource.Body('description')
    #: Extended parameters.
    extended_param = resource.Body('extendParam', type=dict)
    #: Cluster flavors.
    flavor = resource.Body('flavor')
    #: Node network parameters.
    host_network = resource.Body('hostNetwork', type=HostNetworkSpec)
    #: Service forwarding mode
    kube_proxy_mode = resource.Body('kubeProxyMode')
    #: Service CIDR block or the IP address range which the kubernetes
    #: clusterIp must fall within
    service_ip_range = resource.Body('kubernetesSvcIpRange')
    #: Cluster type.
    type = resource.Body('type')
    #: Cluster version ['v1.11.7-r2', 'v1.13.10-r0'].
    version = resource.Body('version')
    #: Eni network parameters.
    eni_network = resource.Body('eniNetwork', type=EniNetworkSpec)


class StatusSpec(_base.StatusSpec):
    # Properties
    #: Access address of the kube-apiserver in the cluster.
    endpoints = resource.Body('endpoints', type=dict)


class Cluster(_base.Resource):
    base_path = '/clusters'

    resources_key = ''
    resource_key = ''

    allow_create = True
    allow_fetch = True
    allow_commit = True
    allow_delete = True
    allow_list = True

    # Properties
    #: specification
    spec = resource.Body('spec', type=ClusterSpec)
    #: Cluster status
    status = resource.Body('status', type=StatusSpec)

    @classmethod
    def new(cls, **kwargs):
        if 'kind' not in kwargs:
            kwargs['kind'] = 'Cluster'
        if 'apiVersion' not in kwargs:
            kwargs['apiVersion'] = 'v3'
        metadata = kwargs.get('metadata', '')
        if 'name' in kwargs and not metadata:
            name = kwargs.pop('name', '')
            kwargs['metadata'] = {
                'name': name
            }
        return cls(_synchronized=False, **kwargs)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
#
"""Backup v3 action implementations"""

from osc_lib import exceptions
from osc_lib import utils
from osc_lib.command import command

from otcextensions.common import sdk_utils
from otcextensions.i18n import _


BACKUP_TYPE_CHOICES = ['auto', 'manual', 'fragment', 'incremental']


_formatters = {
}


def _get_columns(item):
    column_map = {}
    hidden = ['location']
    return sdk_utils.get_osc_show_columns_for_sdk_resource(
        item, column_map, hidden)


class ListBackup(command.Lister):
    _description = _("List database backups")
    column_headers = (
        'ID', 'Name', 'Type', 'Instance Id', 'Size (KB)',
        'Begin time', 'End time'
    )

    columns = (
        'id', 'name', 'type', 'instance_id', 'size',
        'begin_time', 'end_time'
    )

    def get_parser(self, prog_name):
        parser = super(ListBackup, self).get_parser(prog_name)
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('Specify instance ID or Name to get backup list'),
        )
        parser.add_argument(
            '--backup-id',
            metavar='<backup_id>',
            help=_('Specify the backup ID.'),
        )
        parser.add_argument(
            '--backup-type',
            metavar='{' + ','.join(BACKUP_TYPE_CHOICES) + '}',
            choices=BACKUP_TYPE_CHOICES,
            type=lambda s: s.lower(),
            help=_('Specify the backup type.'),
        )
        parser.add_argument(
            '--offset',
            metavar='<offset>',
            type=int,
            help=_('Specify the index position.'),
        )
        parser.add_argument(
            '--limit',
            metavar='<limit>',
            type=int,
            help=_('Specify the limit of resources to be queried.'),
        )
        parser.add_argument(
            '--begin-time',
            metavar='<begin_time>',
            help=_('Specify the start time for obtaining the backup list.'),
        )
        parser.add_argument(
            '--end-time',
            metavar='<end_time>',
            help=_('Specify the end time for obtaining the backup list.'),
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds
        attrs = {}
        args_list = [
            'backup_id', 'backup_type', 'offset', 'limit',
            'begin_time', 'end_time'
        ]
        must_be_together_group = [parsed_args.begin_time, parsed_args.end_time]

        if any(must_be_together_group) and not all(must_be_together_group):
            raise exceptions.CommandError(
                _('`--begin-time` and `--end-time` must be given together')
            )
        for arg in args_list:
            if getattr(parsed_args, arg):
                attrs[arg] = getattr(parsed_args, arg)
        if parsed_args.limit:
            attrs['paginated'] = False

        instance = client.find_instance(parsed_args.instance,
                                        ignore_missing=False)

        data = client.backups(instance=instance, **attrs)

        return (self.column_headers, (utils.get_item_properties(
            s,
            self.columns,
        ) for s in data))


class CreateBackup(command.ShowOne):
    _description = _('Create Database backup')

    def get_parser(self, prog_name):
        parser = super(CreateBackup, self).get_parser(prog_name)
        parser.add_argument(
            'name',
            metavar='<name>',
            help=_('Name for the backup'))
        parser.add_argument(
            'instance',
            metavar='<instance>',
            help=_('ID or Name of the instance to create backup from'))
        parser.add_argument(
            '--description',
            metavar='<description>',
            help=_('Description for the backup'))
        parser.add_argument(
            '--databases',
            metavar='<databases>',
            help=_(
                'Specifies a CSV list of self-built SQL Server'
                'databases that are partially backed up'
                '(Only SQL Server support partial backups.)'))
        parser.add_argument(
            '--wait',
            action='store_true',
            help=('Wait for the instance to become active')
        )
        parser.add_argument(
            '--wait-interval',
            type=int,
            help=_('Interval for checking status')
        )

        return parser

    def take_action(self, parsed_args):
        client = self.app.client_manager.rds

        attrs = {'name': parsed_args.name}
        if parsed_args.description:
            attrs['description'] = parsed_args.description

        instance = client.find_instance(parsed_args.instance,
                                        ignore_missing=False)

        if (parsed_args.databases
                and instance.datastore['type'].lower() == 'sqlserver'):
            attrs['databases'] = []
            databases = parsed_args.databases
            databases = databases.split(",")
            for db_name in databases:
                attrs['databases'].append({'name': db_name})
        else:
            # Do we want to raise error otherwise?
            pass

        obj = client.create_backup(instance=instance,
                                   **attrs)

        if parsed_args.wait:
            wait_args = {}
            if parsed_args.wait_interval:
                wait_args['interval'] = parsed_args.wait_interval
            obj = client.wait_for_backup(obj, **wait_args)

        display_columns, columns = _get_columns(obj)
        data = utils.get_item_properties(obj, columns,
                                         formatters=_formatters)

        return (display_columns, data)


class DeleteBackup(command.Command):
    _description = _('Delete Backup')

    def get_parser(self, prog_name):
        parser = super(DeleteBackup, self).get_parser(prog_name)
        parser.add_argument('backup',
                            metavar='<backup_id>',
                            nargs='+',
                            help=_('ID of the backup'))
        return parser

    def take_action(self, parsed_args):

        if parsed_args.backup:
            client = self.app.client_manager.rds
            for bck in parsed_args.backup:
                client.delete_backup(backup=bck, ignore_missing=False)


class ListBackupDownloadLinks(command.Lister):
    _description = _('List Backup Download Links')

    column_headers = ('Size (KB)', 'URL', 'Expires at')
    columns = ('size', 'download_link', 'expires_at')

    def get_parser(self, prog_name):
        parser = super(ListBackupDownloadLinks, self).get_parser(prog_name)
        parser.add_argument('backup_id',
                            metavar='<backup_id>',
                            help=_('ID of the backup'))
        return parser

    def take_action(self, parsed_args):

        client = self.app.client_manager.rds
        data = client.backup_download_links(backup_id=parsed_args.backup_id)
        return (self.column_headers, (utils.get_item_properties(
            s,
            self.columns,
            formatters={}
        ) for s in data))



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack.tests.unit import base
from otcextensions.sdk.apig.v2 import acl_policy

EXAMPLE_AC_POLICY = {
    'gateway_id': 'gateway-67890',
    'id': 'acl-123',
    'acl_name': 'Test ACL',
    'acl_type': 'IP',
    'acl_value': '192.168.0.1',
    'entity_type': 'APP',
    'update_time': '2025-02-07T13:00:00Z',
    'bind_num': 3,
    'error_code': '',
    'error_msg': ''
}


class TestAcPolicy(base.TestCase):

    def test_basic(self):
        sot = acl_policy.AclPolicy()
        self.assertEqual('apigw/instances/%(gateway_id)s/acls', sot.base_path)
        self.assertEqual('acls', sot.resources_key)
        self.assertTrue(sot.allow_create)
        self.assertTrue(sot.allow_commit)
        self.assertTrue(sot.allow_delete)
        self.assertTrue(sot.allow_list)
        self.assertTrue(sot.allow_fetch)

    def test_make_it(self):
        sot = acl_policy.AclPolicy(**EXAMPLE_AC_POLICY)
        self.assertEqual(EXAMPLE_AC_POLICY['gateway_id'], sot.gateway_id)
        self.assertEqual(EXAMPLE_AC_POLICY['id'], sot.id)
        self.assertEqual(EXAMPLE_AC_POLICY['acl_name'], sot.acl_name)
        self.assertEqual(EXAMPLE_AC_POLICY['acl_type'], sot.acl_type)
        self.assertEqual(EXAMPLE_AC_POLICY['acl_value'], sot.acl_value)
        self.assertEqual(EXAMPLE_AC_POLICY['entity_type'], sot.entity_type)
        self.assertEqual(EXAMPLE_AC_POLICY['update_time'], sot.update_time)
        self.assertEqual(EXAMPLE_AC_POLICY['bind_num'], sot.bind_num)
        self.assertEqual(EXAMPLE_AC_POLICY['error_code'], sot.error_code)
        self.assertEqual(EXAMPLE_AC_POLICY['error_msg'], sot.error_msg)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.

from openstack.tests.unit import base

from otcextensions.sdk.function_graph.v2 import function_invocation


EXAMPLE = {
    'func_urn': 'test-function',
    'attrs': {'a': 'b'}
}


class TestFunctionInvocation(base.TestCase):

    def test_basic(self):
        sot = function_invocation.FunctionInvocation()
        path = '/fgs/functions/%(func_urn)s'
        self.assertEqual(path, sot.base_path)
        self.assertTrue(sot.allow_create)

    def test_make_it(self):
        sot = function_invocation.FunctionInvocation(**EXAMPLE)
        self.assertEqual(EXAMPLE['func_urn'], sot.func_urn)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
# import six
from openstack import exceptions
from openstack import resource


class AppConsumption(resource.Resource):
    base_path = '/apps/%(app_name)s/streams/%(stream_name)s'

    resources_key = 'partition_consuming_states'

    allow_list = True

    #: Name of the app to be queried.
    app_name = resource.URI('app_name')
    #: Name of the stream to be queried.
    stream_name = resource.URI('stream_name')

    _query_mapping = resource.QueryParameters(
        'checkpoint_type', 'start_partition_id', 'limit'
    )

    # Properties
    #: Partition Id.
    partition_id = resource.Body('partition_id')
    #: Partition Sequence Number
    sequence_number = resource.Body('sequence_number')
    #: Partition data latest offset.
    latest_offset = resource.Body('latest_offset', type=int)
    #: Partition data earliest offset.
    earliest_offset = resource.Body('earliest_offset', type=int)
    #: Type of the checkpoint.
    #:  \nLAST_READ: Only sequence numbers are recorded in databases.
    checkpoint_type = resource.Body('checkpoint_type')
    #: Partition Status.
    status = resource.Body('status')


class App(resource.Resource):
    base_path = '/apps'

    resources_key = 'apps'

    allow_create = True
    allow_list = True
    allow_delete = True
    allow_fetch = True

    _query_mapping = resource.QueryParameters(
        'limit', 'start_app_name', 'stream_name'
    )

    # Properties
    #: Unique identifier of the app.
    app_id = resource.Body('app_id', alternate_id=True)
    #: List of associated streams.
    commit_checkpoint_stream_names = \
        resource.Body('commit_checkpoint_stream_names', type=list)
    #: Timestamp at which the DIS app was created.
    created_at = resource.Body('create_time', type=int)
    #: App name.
    name = resource.Body('app_name')
    #: Stream Name
    stream_name = resource.Body('stream_name')

    @classmethod
    def list(cls, session, paginated=True, base_path=None,
             allow_unknown_params=False, **params):

        if not cls.allow_list:
            raise exceptions.MethodNotSupported(cls, "list")
        session = cls._get_session(session)
        microversion = cls._get_microversion(session)

        if base_path is None:
            base_path = cls.base_path
        params = cls._query_mapping._validate(
            params, base_path=base_path,
            allow_unknown_params=allow_unknown_params)

        query_params = cls._query_mapping._transpose(params, cls)
        uri = base_path % params

        while uri:
            # Copy query_params due to weird mock unittest interactions
            response = session.get(
                uri,
                headers={"Accept": "application/json"},
                params=query_params.copy(),
                microversion=microversion)
            exceptions.raise_from_response(response)
            data = response.json()

            if cls.resources_key:
                resources = data[cls.resources_key]
            else:
                resources = data

            if not isinstance(resources, list):
                resources = [resources]

            for raw_resource in resources:
                value = cls.existing(
                    microversion=microversion,
                    connection=session._get_connection(),
                    **raw_resource)
                yield value

            if resources and paginated:
                uri, next_params = cls._get_next_link(
                    uri, response, data)
                query_params.update(next_params)
            else:
                return

    @classmethod
    def _get_next_link(cls, uri, response, data):
        # RDS service pagination. Returns query for the next page
        next_link = None
        params = {}
        if 'has_more_app' in data and data['has_more_app']:
            next_link = uri
            params['start_app_name'] = data['apps'][-1]['app_name']
        else:
            next_link = None
        query_params = cls._query_mapping._transpose(params, cls)
        return next_link, query_params



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource
from openstack import exceptions


class FunctionInvocation(resource.Resource):
    base_path = '/fgs/functions/%(func_urn)s'
    # Capabilities
    allow_create = True

    _query_mapping = resource.QueryParameters('func_urn')

    # Properties
    func_urn = resource.URI('func_urn', type=str)

    # Attributes
    invoke_summary = resource.Header('X-Cff-Invoke-Summary', type=str)
    cff_request_id = resource.Header('X-Cff-Request-Id', type=str)
    request_id = resource.Body('request-id', type=str)
    function_log = resource.Header('X-Cff-Function-Log', type=str)
    billing_duration = resource.Header('X-CFF-Billing-Duration', type=str)
    response_version = resource.Header('X-Cff-Response-Version', type=str)
    err_code = resource.Header('X-Func-Err-Code', type=str)
    err = resource.Header('X-Is-Func-Err', type=str)

    def _invocation(self, session, action, attrs: dict):
        """Function Execution
        """
        url = (self.base_path % self._uri.attributes) + f'/{action}'
        response = session.post(url, json=str(attrs).replace("'", '"'))
        exceptions.raise_from_response(response)
        self._translate_response(response)
        return self



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
import mock

from keystoneauth1 import adapter

from openstack.tests.unit import base

from otcextensions.sdk.dcs.v1 import statistic

FAKE_ID = "68d5745e-6af2-40e4-945d-fe449be00148"
EXAMPLE = {
    "keys": 0,
    "instance_id": "e008652d-18e0-43ff-924e-072261e0372a",
    "used_memory": 0,
    "max_memory": 460,
    "cmd_get_count": 0,
    "cmd_set_count": 0,
    "used_cpu": "0.0",
    "input_kbps": "0.03",
    "output_kbps": "1.19"
}


class TestStatistics(base.TestCase):

    def setUp(self):
        super(TestStatistics, self).setUp()
        self.sess = mock.Mock(spec=adapter.Adapter)
        self.sess.put = mock.Mock()

    def test_basic(self):
        sot = statistic.Statistic()

        self.assertEqual('/instances/statistic', sot.base_path)

        self.assertTrue(sot.allow_list)

    def test_make_it(self):

        sot = statistic.Statistic(**EXAMPLE)
        self.assertEqual(EXAMPLE['instance_id'], sot.id)
        self.assertEqual(EXAMPLE['keys'], sot.keys)
        self.assertEqual(EXAMPLE['used_memory'], sot.used_memory)
        self.assertEqual(EXAMPLE['max_memory'], sot.max_memory)
        self.assertEqual(EXAMPLE['cmd_get_count'], sot.cmd_get_count)
        self.assertEqual(EXAMPLE['cmd_set_count'], sot.cmd_set_count)
        self.assertEqual(EXAMPLE['used_cpu'], sot.used_cpu)
        self.assertEqual(EXAMPLE['input_kbps'], sot.input_kbps)
        self.assertEqual(EXAMPLE['output_kbps'], sot.output_kbps)



# Licensed under the Apache License, Version 2.0 (the "License"); you may
# not use this file except in compliance with the License. You may obtain
# a copy of the License at
#
#      http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
# WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
# License for the specific language governing permissions and limitations
# under the License.
from openstack import resource


class DirectConnectEndpointGroup(resource.Resource):

    base_path = '/dcaas/dc-endpoint-groups'
    resource_key = 'dc_endpoint_group'
    resources_key = 'dc_endpoint_groups'

    # capabilities
    allow_list = True
    allow_fetch = True
    allow_create = True
    allow_commit = True
    allow_delete = True

    _query_mapping = resource.QueryParameters(
        'id', 'project_id', 'name', 'description', 'endpoints', 'type',
        projrct_id='tenant_id'
    )

    # Properties
    #: The ID of the Direct Connect Endpoint Group.
    id = resource.Body('id')
    #: The name of the Direct Connect Endpoint Group.
    name = resource.Body('name')
    #: Provides supplementary information about the Direct Connect Endpoint
    # Group.
    description = resource.Body('description')
    #: The list of the endpoints in a Direct Connect Endpoint Group
    endpoints = resource.Body('endpoints', type=list)
    #: The type of the Direct Connect Endpoints. The value can only be cidr.
    type = resource.Body('type')
    #: The project ID.
    project_id = resource.Body('tenant_id')
